////////////////////////////////////////////////////////////////
// Autor : Regedit
// Data  : 2003-06-16
// Opis  : Szkielet aplikacji DirectX
////////////////////////////////////////////////////////////////

#define STRICT

#include <windows.h>
#include <d3dx9.h>
#include "Error.h"
#include "App.h"

// Zmienna globalna, dla WndProc
static CApplication* g_pApplication = NULL;


// Procedura okna
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	return g_pApplication->MsgProc(hWnd, uMsg, wParam, lParam);
}

CApplication::CApplication()
{
	g_pApplication = this;
	m_bWindowed = false;
	m_bActive = false;
	m_bDeviceLost = false;
	m_bMinimized = false;
	m_bDeviceObjectsInited = false;
	m_bDeviceObjectsRestored = false;
	m_bShowCursor = false;
	m_bClosing = false;

	m_hWnd = NULL;
	m_pD3D = NULL;
	m_pDevice = NULL;

	m_dwWidth = 640;
	m_dwHeight = 480;

	m_lpszClassName = "PXDXAPP";
	m_lpszWindowName = "";

	Pause(true);
}

void CApplication::Pause(bool bPause)
{
	static int PauseCount = 0;

	PauseCount += (bPause ? +1 : -1);
	m_bActive = (PauseCount ? false : true );

	// Timer stop
/*	if ((PauseCount == 1) && bPause)
		;
	// Timer start
	else */ if (PauseCount == 0)
	{
		m_dwLastCzas = GetTickCount();
		m_dwLastFrameCzas = m_dwLastCzas;
		m_dwFrameCount = 0;
		m_dwFrameRate = 0;
	}
}

bool CApplication::Create(HINSTANCE hInstance, char* lpszCmdLine)
{
	// Parse parameters
	ParseParameters(lpszCmdLine);

	// Confirm
	if (!Confirm())
		return false;

	// Create the Direct3D object
	m_pD3D = Direct3DCreate9(D3D_SDK_VERSION);
	if (m_pD3D == NULL)
		throw CError(ERRTYPE_NORMAL, "Nie mona utworzy obiektu Direct3D 9.", __FILE__, __LINE__);

	// Registering window class
	WNDCLASS wndClass = {CS_DBLCLKS, WndProc, 0, 0, hInstance, NULL,
		LoadCursor(NULL, IDC_ARROW), NULL, NULL, m_lpszClassName};
	if (RegisterClass(&wndClass) == 0)
		throw CError(ERRTYPE_WIN32API, "Nie mona zarejestrowa klasy okna", __FILE__, __LINE__);

	// * Creating window *
	// - Styles...
	DWORD dwStyle = WS_VISIBLE;
	DWORD dwExStyle = 0;
	RECT rect = {0, 0, m_dwWidth, m_dwHeight};
	if (m_bWindowed)
	{
		dwStyle |= WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX;
		AdjustWindowRectEx(&rect, dwStyle, false, dwExStyle);
	}
	else
	{
		dwStyle |= WS_POPUP;
		dwExStyle = WS_EX_TOPMOST;
	}

	// - Creating...
	m_hWnd = CreateWindowEx(dwExStyle, m_lpszClassName, m_lpszWindowName, dwStyle,
		CW_USEDEFAULT, CW_USEDEFAULT,
		(rect.right-rect.left), (rect.bottom-rect.top), NULL, NULL, hInstance, 0);

	// - Error handling...
	if (m_hWnd == NULL)
		throw CError(ERRTYPE_WIN32API, "Nie mona utworzy okna.", __FILE__, __LINE__);

	// OneTimeSceneInit
	try
	{
		OneTimeSceneInit();
	}
	catch(...)
	{
		m_pD3D->Release();
		m_pD3D = NULL;
		throw;
	}

	// Initialize3DEnvironment
	try
	{
		Initialize3DEnvironment();
	}
	catch(...)
	{
		m_pD3D->Release();
		m_pD3D = NULL;
		throw;
	}

	// OK, ready to start!
	Pause(false);

	return true;
}

void CApplication::ParseParameters(char* lpszCmdLine)
{
	bool quot = false;
	int index = 0;
	SIZE_T len = strlen(lpszCmdLine);
	char str[MAX_PATH];
	strcpy(str, "");

	for (SIZE_T i = 0; i <= len; i++)
	{
		if ( (i == len) || ((lpszCmdLine[i] == ' ') && (!quot)) )
		{
			str[index] = '\0';
			ParseParameter(str);
			strcpy(str, "");
			index = 0;
		}
		else if (lpszCmdLine[i] == '"')
			quot = !quot;
		else
			if (index < MAX_PATH-1)
				str[index++] = lpszCmdLine[i];
	}
}

void CApplication::ParseParameter(char *lpszParameter)
{
	// Window
	if ( (strcmp(lpszParameter, "-w") == 0) || (strcmp(lpszParameter, "--window") == 0) )
        m_bWindowed = true;
}

void CApplication::Initialize3DEnvironment()
{
	// Cursor
	if (!(m_bWindowed || m_bShowCursor))
		ShowCursor(false);

	// Utworzenie urzdzenia
	ZeroMemory(&m_PresentParameters, sizeof(m_PresentParameters));
	m_PresentParameters.BackBufferWidth = m_dwWidth;
	m_PresentParameters.BackBufferHeight = m_dwHeight;
	m_PresentParameters.BackBufferCount = 1;
	m_PresentParameters.MultiSampleType = D3DMULTISAMPLE_NONE;
	m_PresentParameters.SwapEffect = D3DSWAPEFFECT_DISCARD;
	m_PresentParameters.hDeviceWindow = m_hWnd;
	m_PresentParameters.EnableAutoDepthStencil = false;
	m_PresentParameters.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
	m_PresentParameters.PresentationInterval = D3DPRESENT_INTERVAL_DEFAULT;

	if (m_bWindowed)
	{
		D3DDISPLAYMODE d3ddm;
		m_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm);
		m_PresentParameters.BackBufferFormat = d3ddm.Format;
		m_PresentParameters.Windowed = true;
	}
	else
	{
		m_PresentParameters.BackBufferFormat = D3DFMT_A8R8G8B8;
		m_PresentParameters.Windowed = false;
	}

	HRESULT hr;

	if (FAILED(hr = m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd, D3DCREATE_HARDWARE_VERTEXPROCESSING, &m_PresentParameters, &m_pDevice)))
		if (FAILED(hr = m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &m_PresentParameters, &m_pDevice)))
			throw CError(ERRTYPE_DIRECTX, "Nie mona utworzy urzdzenia", __FILE__, __LINE__, hr);

	try
	{
		InitDeviceObjects();
	}
	catch(...)
	{
		DeleteDeviceObjects();
		Cleanup3DEnvironment();
		throw;
	}

	m_bDeviceObjectsInited = true;

	try
	{
		RestoreDeviceObjects();
	}
	catch(...)
	{
		InvalidateDeviceObjects();
		throw;
	}

	m_bDeviceObjectsRestored = true;
}

void CApplication::Reset3DEnvironment()
{
	HRESULT hr;

	if (m_bDeviceObjectsRestored)
	{
		m_bDeviceObjectsRestored = false;
		InvalidateDeviceObjects();
	}

	if (FAILED(hr = m_pDevice->Reset(&m_PresentParameters)))
		throw CError(ERRTYPE_DIRECTX, "Bd podczas resetowania urzdzenia", __FILE__, __LINE__, hr);

	try
	{
		RestoreDeviceObjects();
	}
	catch(...)
	{
		InvalidateDeviceObjects();
		throw;
	}

	m_bDeviceObjectsRestored = true;
}

void CApplication::Cleanup3DEnvironment()
{
	if (m_pDevice != NULL)
	{
		if (m_bDeviceObjectsRestored)
		{
			m_bDeviceObjectsRestored = false;
			InvalidateDeviceObjects();
		}
		if (m_bDeviceObjectsInited)
		{
			m_bDeviceObjectsInited = false;
			DeleteDeviceObjects();
		}

		if (m_pDevice->Release() > 0)
			Beep(1500, 333);
		m_pDevice = NULL;
	}

	// Cursor
	if (!(m_bWindowed || m_bShowCursor))
		ShowCursor(true);
}

void CApplication::Render3DEnvironment()
{
	HRESULT hr;

	// Utracone urzdzenie
	if (m_bDeviceLost)
	{
		if (FAILED(hr = m_pDevice->TestCooperativeLevel()))
		{
			if (D3DERR_DEVICELOST == hr)
				return;

			if (D3DERR_DEVICENOTRESET == hr)
			{
				Reset3DEnvironment();
				return;
			}

			throw CError(ERRTYPE_DIRECTX, "Bd podczas testowania poziomu kooperatywnoci urzdzenia", __FILE__, __LINE__, hr);
		}
		m_bDeviceLost = false;
	}

	// Czas
	DWORD dwCzas = GetTickCount();
	DWORD dwDeltaCzas = dwCzas - m_dwLastCzas;

	// FPS
	m_dwFrameCount++;
	if (dwCzas > m_dwLastFrameCzas + 1000)
	{
		m_dwFrameRate = m_dwFrameCount;
		m_dwFrameCount = 0;
		m_dwLastFrameCzas += 1000;
	}

	// FrameMove() i Render()
	FrameMove(dwCzas, dwDeltaCzas);
	Render();

	// Present
	hr = m_pDevice->Present(NULL, NULL, NULL, NULL);
	if (D3DERR_DEVICELOST == hr)
		m_bDeviceLost = true;

	// Koniec
	m_dwLastCzas = dwCzas;
}

LRESULT CApplication::MsgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_PAINT:
		if (m_pDevice && !m_bActive && m_bWindowed &&
			m_bDeviceObjectsInited && m_bDeviceObjectsRestored)
		{
			Render();
			m_pDevice->Present(NULL, NULL, NULL, NULL);
		}
		break;

	case WM_ENTERSIZEMOVE:
		Pause(true);
		break;

	case WM_SIZE:
		if (SIZE_MINIMIZED == wParam)
		{
			Pause(true);
			m_bMinimized = true;
		}
		else if (SIZE_RESTORED == wParam)
		{
			if (m_bMinimized)
			{
				Pause(false);
				m_bMinimized = false;
			}
		}
		break;

	case WM_EXITSIZEMOVE:
		Pause(false);
		break;

	case WM_ENTERMENULOOP:
		Pause(true);
		break;

	case WM_EXITMENULOOP:
		Pause(false);
		break;

	case WM_CLOSE:
		m_pD3D->Release();
		m_pD3D = NULL;
		FinalCleanup();
		PostQuitMessage(0);
		return 0;
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

int CApplication::Run()
{
	bool bGotMsg;
	MSG msg;
	msg.message = WM_NULL;
	PeekMessage(&msg, NULL, 0U, 0U, PM_NOREMOVE);

	while ( (WM_QUIT != msg.message) && !m_bClosing )
	{
		if (m_bActive)
			bGotMsg = (PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE) != 0);
		else
			bGotMsg = (GetMessage(&msg, NULL, 0U, 0U) != 0);

		if (bGotMsg)
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			if (m_bDeviceLost)
			{
				Sleep(100);
			}
			if (m_bActive)
			{
				try
				{
					Render3DEnvironment();
				}
				catch(...)
				{
					SendMessage(m_hWnd, WM_CLOSE, 0, 0);
				}
			}
		}
	}

	Cleanup3DEnvironment();
	return (int)msg.wParam;
}