////////////////////////////////////////////////////////////////
// Autor : Regedit
// Data  : 2003-06-16
// Opis  : Klasa wyjtku do obsugi bdw
////////////////////////////////////////////////////////////////

#include <windows.h>
#include <lmcons.h> // ULEN
#include <d3dx9.h>
#include <dxerr9.h>
#include <stdio.h> // sprintf
#include "Error.h"

// Licznik bdw
int g_iIndex = 1;

// * Konstruktor klasy wyjtku *
// Sposb uycia
// 1. Dla bdw zwykych:	CError(ERRTYPE_NORMAL, "Tre", __FILE__, __LINE__);
// 2. Dla bdw Win32API:	CError(ERRTYPE_WIN32API, "Tre", __FILE__, __LINE__);
// 3. Dla bdw DirectX:	CError(ERRTYPE_DIRECTX, "Tre", __FILE__, __LINE__, hr);
CError::CError(int iType, char* lpszMessage, char* lpszSrcFile, int iSrcLine, HRESULT hr)
{
	m_iType = iType;
	m_lpszMessage = lpszMessage;
	m_lpszSrcFile = lpszSrcFile;
	m_iSrcLine = iSrcLine;
	switch (iType)
	{
	case ERRTYPE_WIN32API:
		m_dwWin32APICode = GetLastError();
		break;
	case ERRTYPE_DIRECTX:
		m_hrDirectXCode = hr;
	}

	Log();
}

void CError::Log()
{
	if (LogOpen())
	{
		LogWriteln("rodowisko");
		LogWriteTime();
		LogWriteComputerName();
		LogWriteUserName();
		LogWriteFileName();

		LogWriteSourceInfo();
		LogWriteErrorDetails();

		switch(m_iType)
		{
		case ERRTYPE_WIN32API:
			LogWriteWin32APIDetails();
			break;
		case ERRTYPE_DIRECTX:
			LogWriteDirectXDetails();
			break;
		}

		LogWriteln();
		LogClose();
	}
}

bool CError::LogOpen()
{
	if ((m_hFile = CreateFile(ERR_LOGFILE, GENERIC_WRITE, 0, NULL, OPEN_ALWAYS,
		FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE)
		return false;
	SetFilePointer(m_hFile, 0, NULL, FILE_END);
	return true;
}

void CError::LogClose()
{
	CloseHandle(m_hFile);
}

void CError::LogWriteln()
{
	LogWriteln("");
}

void CError::LogWriteln(const char* lpsz)
{
	DWORD dupa;
	WriteFile(m_hFile, lpsz, (DWORD)strlen(lpsz), &dupa, NULL);
	WriteFile(m_hFile, CRLF, (DWORD)strlen(CRLF), &dupa, NULL);
}

void CError::LogWriteln(const char* lpsz1, const char* lpsz2)
{
	char* s = new char[strlen(lpsz1)+strlen(lpsz2)+1];
	strcpy(s, lpsz1);
	strcat(s, lpsz2);
	LogWriteln(s);
	delete [] s;
}

void CError::LogWriteln(const char* lpsz, int i)
{
	char s[16];
	sprintf(s, "%i", i);
	LogWriteln(lpsz, s);
}

void CError::LogWriteln(const char* lpsz, DWORD dw)
{
	char s[16];
	sprintf(s, "%u", dw);
	LogWriteln(lpsz, s);
}

void CError::LogWriteTime()
{
	SYSTEMTIME st;
	GetSystemTime(&st);
	char s[32];
	sprintf(s, "%.4u-%.2u-%.2u %.2u:%.2u:%.2u", st.wYear, st.wMonth, st.wDay,
		st.wHour, st.wMinute, st.wSecond);
	LogWriteln(ERROR_LOGSTR_DATE, s);
}

void CError::LogWriteComputerName()
{
	const DWORD size = MAX_COMPUTERNAME_LENGTH+1;
	DWORD dupa = size;
	char lpsz[size];
	if (GetComputerName(lpsz, &dupa))
		LogWriteln(ERROR_LOGSTR_COMPUTER, lpsz);
}

void CError::LogWriteUserName()
{
	const DWORD size = UNLEN+1;
	DWORD dupa = size;
	char lpsz[size];
	if (GetUserName(lpsz, &dupa))
		LogWriteln(ERROR_LOGSTR_USER, lpsz);
}

void CError::LogWriteFileName()
{
	const DWORD size = MAX_PATH;
	DWORD dupa = size;
	char lpsz[size];
	if (GetModuleFileName(NULL, lpsz, dupa))
		LogWriteln(ERROR_LOGSTR_FILE, lpsz);
}

void CError::LogWriteSourceInfo()
{
	LogWriteln("rdo");
	// Plik
	LogWriteln(ERROR_LOGSTR_FILE, m_lpszSrcFile);
	// Wiersz
	LogWriteln(ERROR_LOGSTR_ROW, m_iSrcLine);
}

void CError::LogWriteErrorDetails()
{
	LogWriteln("Bd");
	// Numer
	LogWriteln(ERROR_LOGSTR_INDEX, g_iIndex++);
	// Typ
	char* str;
	switch (m_iType)
	{
	case ERRTYPE_NORMAL:
		str = "Normalny";
		break;
	case ERRTYPE_WIN32API:
		str = "Win32API";
		break;
	case ERRTYPE_DIRECTX:
		str = "DirectX";
		break;
	default:
		str = "Nieznany";
	}
	LogWriteln(ERROR_LOGSTR_TYPE, str);
	// Komunikat
	LogWriteln(ERROR_LOGSTR_MESSAGE, m_lpszMessage);
}

void CError::LogWriteWin32APIDetails()
{
	LogWriteln("Win32API");
	// Kod
	LogWriteln(ERROR_LOGSTR_CODE, m_dwWin32APICode);
	// Komunikat
	char *lpszMessage = NULL;
	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER |
		FORMAT_MESSAGE_FROM_SYSTEM |
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL, m_dwWin32APICode, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPSTR)&lpszMessage, 0, NULL);
	LogWriteln(ERROR_LOGSTR_MESSAGE, lpszMessage);
	LocalFree(lpszMessage);
}

void CError::LogWriteDirectXDetails()
{
	LogWriteln("DirectX");
	// Kod
	LogWriteln(ERROR_LOGSTR_CODE, m_hrDirectXCode);
	// Komunikat
	LogWriteln(ERROR_LOGSTR_MESSAGE, DXGetErrorString9(m_hrDirectXCode));
	// Opis
	LogWriteln(ERROR_LOGSTR_DESC, DXGetErrorDescription9(m_hrDirectXCode));
}