////////////////////////////////////////////////////////////////
// Autor : Regedit
// Data  : 2003-06-16
// Opis  : Obiekt - singleton to zarzdzania grafik
////////////////////////////////////////////////////////////////
#include <windows.h>
#include <d3dx9.h>
#include "Error.h"
#include "Scr.h"
#include "Gfx.h"

////////////////////////////////////////////////////////////////
// Elementy globalne

LPDIRECT3DDEVICE9 g_pDevice;
CGfx* g_pGfx;
int g_iCursorX;
int g_iCursorY;

////////////////////////////////////////////////////////////////
// CGfx

CGfx::CGfx()
{
	m_texCursor = NULL;
	m_texDesktop = NULL;
	m_texLogo = NULL;
	Font = NULL;
}

CGfx::~CGfx()
{
}

void CGfx::InitDeviceObjects()
{
	HRESULT hr;

	// Kursor
	if (FAILED(hr = D3DXCreateTextureFromFile(g_pDevice, FILE_CURSOR, &m_texCursor)))
	{
		char s[MAX_PATH] = "Nie mona wczyta kursora z pliku ";
		strcat(s, FILE_CURSOR);
		throw CError(ERRTYPE_DIRECTX, s, __FILE__, __LINE__, hr);
	}

	// Pulpit
	if (FAILED(hr = D3DXCreateTextureFromFile(g_pDevice, FILE_DESKTOP, &m_texDesktop)))
	{
		char s[MAX_PATH] = "Nie mona wczyta obrazu pulpitu z pliku ";
		strcat(s, FILE_DESKTOP);
		throw CError(ERRTYPE_DIRECTX, s, __FILE__, __LINE__, hr);
	}

	// Logo
	if (FAILED(hr = D3DXCreateTextureFromFile(g_pDevice, FILE_LOGO, &m_texLogo)))
	{
		char s[MAX_PATH] = "Nie mona wczyta logo z pliku ";
		strcat(s, FILE_LOGO);
		throw CError(ERRTYPE_DIRECTX, s, __FILE__, __LINE__, hr);
	}

	// Przyciski
	if (FAILED(hr = D3DXCreateTextureFromFile(g_pDevice, FILE_BUTTONS, &m_texButtons)))
	{
		char s[MAX_PATH] = "Nie mona wczyta obrazw przyciskw z pliku ";
		strcat(s, FILE_BUTTONS);
		throw CError(ERRTYPE_DIRECTX, s, __FILE__, __LINE__, hr);
	}
}

void CGfx::RestoreDeviceObjects()
{
	HRESULT hr;

	LOGFONT lf = {16, 0, 0, 0, FW_BOLD, false, false, false,
		EASTEUROPE_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
		DEFAULT_QUALITY, FF_DONTCARE, "Verdana"};
	if (FAILED(hr = D3DXCreateFontIndirect(g_pDevice, &lf, &Font)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona utworzy czcionki", __FILE__, __LINE__, hr);
}

void CGfx::InvalidateDeviceObjects()
{
	if (Font)
	{
		Font->Release();
		Font = NULL;
	}

}

void CGfx::DeleteDeviceObjects()
{
	if (m_texButtons)
	{
		m_texButtons->Release();
		m_texButtons = NULL;
	}

	if (m_texLogo)
	{
		m_texLogo->Release();
		m_texLogo = NULL;
	}

	if (m_texDesktop)
	{
		m_texDesktop->Release();
		m_texDesktop = NULL;
	}

	if (m_texCursor)
	{
		m_texCursor->Release();
		m_texCursor = NULL;
	}
}

void CGfx::RenderCursor()
{
	DrawTexture(static_cast<float>(g_iCursorX),
		static_cast<float>(g_iCursorY),
		static_cast<float>(g_iCursorX) + 32.0f,
		static_cast<float>(g_iCursorY) + 32.0f,
		m_texCursor);
}

void CGfx::DrawTexture(float x1, float y1, float x2, float y2, LPDIRECT3DTEXTURE9 pTexture)
{
	MYVERTEX v1, v[4];

	v1.rhw = 1.0f;
	v1.color = 0xffffffff;

	v[0] = v1;
	v[0].pos = D3DXVECTOR3(x1, y1, 0.0f);
	v[0].tx = 0.0f;
	v[0].ty = 0.0f;
	v[1] = v1;
	v[1].pos = D3DXVECTOR3(x1, y2, 0.0f);
	v[1].tx = 0.0f;
	v[1].ty = 1.0f;
	v[2] = v1;
	v[2].pos = D3DXVECTOR3(x2, y1, 0.0f);
	v[2].tx = 1.0f;
	v[2].ty = 0.0f;
	v[3] = v1;
	v[3].pos = D3DXVECTOR3(x2, y2, 0.0f);
	v[3].tx = 1.0f;
	v[3].ty = 1.0f;

	g_pDevice->SetTexture(0, pTexture);
	g_pDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
	g_pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	g_pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);

	g_pDevice->SetFVF(FVF_MYVERTEX);
	g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, v, sizeof(MYVERTEX));
}

void CGfx::RenderDesktop(float Width, float Height, float ScrollX, float ScrollY)
{
	MYVERTEX v[4];

	v[0].pos = D3DXVECTOR3(-ScrollX, -ScrollY, 0.0f);
	v[0].rhw = 1.0f;
	v[0].color = 0xffffffff;
	v[0].tx = 0.0f;
	v[0].ty = 0.0f;
	v[1].pos = D3DXVECTOR3(-ScrollX, Height-ScrollY, 0.0f);
	v[1].rhw = 1.0f;
	v[1].color = 0xffffffff;
	v[1].tx = 0.0f;
	v[1].ty = Height / DESKTOPTEXTUREHEIGHT;
	v[2].pos = D3DXVECTOR3(Width-ScrollX, -ScrollY, 0.0f);
	v[2].rhw = 1.0f;
	v[2].color = 0xffffffff;
	v[2].tx = Width / DESKTOPTEXTUREWIDTH;
	v[2].ty = 0.0f;
	v[3].pos = D3DXVECTOR3(Width-ScrollX, Height-ScrollY, 0.0f);
	v[3].rhw = 1.0f;
	v[3].color = 0xffffffff;
	v[3].tx = Width / DESKTOPTEXTUREWIDTH;
	v[3].ty = Height / DESKTOPTEXTUREHEIGHT;

	g_pDevice->SetTexture(0, m_texDesktop);
	g_pDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
	g_pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	g_pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);

	g_pDevice->SetFVF(FVF_MYVERTEX);
	g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, v, sizeof(MYVERTEX));
}

void CGfx::RenderLogo()
{
	MYVERTEX v1, v[4];

	v1.rhw = 1.0f;
	v1.color = 0xffffffff;

	v[0] = v1;
	v[0].pos = D3DXVECTOR3(SCRWIDTH/2.0f-256.0f, SCRHEIGHT/2.0f-128.0f, 0.0f);
	v[0].tx = 0.0f; v[0].ty = 0.25f;
	v[1] = v1;
	v[1].pos = D3DXVECTOR3(SCRWIDTH/2.0f-256.0f, SCRHEIGHT/2.0f+128.0f, 0.0f);
	v[1].tx = 0.0f; v[1].ty = 0.75f;
	v[2] = v1;
	v[2].pos = D3DXVECTOR3(SCRWIDTH/2.0f+256.0f, SCRHEIGHT/2.0f-128.0f, 0.0f);
	v[2].tx = 1.0f; v[2].ty = 0.25f;
	v[3] = v1;
	v[3].pos = D3DXVECTOR3(SCRWIDTH/2.0f+256.0f, SCRHEIGHT/2.0f+128.0f, 0.0f);
	v[3].tx = 1.0f; v[3].ty = 0.75f;

	g_pDevice->SetTexture(0, m_texLogo);
	g_pDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
	g_pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	g_pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);

	g_pDevice->SetFVF(FVF_MYVERTEX);
	g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, v, sizeof(MYVERTEX));
}