////////////////////////////////////////////////////////////////
// Autor : Regedit
// Data  : 2003-06-16
// Opis  : Program gwny
////////////////////////////////////////////////////////////////
#define STRICT

#include <windows.h>
#include <stdio.h> // sprintf
#include <stdlib.h> // atoi
#include <vector>
#include <string>
#include <d3dx9.h>
#include <il.h>
#include <zlib.h>
#include <arrays.h>
#include "Error.h"
#include "App.h"
#include "Gfx.h"
#include "Scr.h"
#include "intro.h"
#include "game.h"
#include "Menu.h"
#include "Puzzle.h"

//-----------------------------------------------------------------------------
// Global access to the app (needed for the global WndProc())
//-----------------------------------------------------------------------------
CMyApplication* g_pApplication = NULL;
HINSTANCE g_hInstance = NULL;

//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything, and goes into a
//       message-processing loop. Idle time is used to render the scene.
//-----------------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, char* lpszCmdLine, int)
{
	srand(GetTickCount());
	CMyApplication Application;

	g_pApplication = &Application;
	g_hInstance = hInstance;

	int result;
	try
	{
		if (Application.Create(hInstance, lpszCmdLine))
			result = Application.Run();
		else
			result = 0;
	}
	catch(...)
	{
		return 0;
	}

	return result;
}

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor.   Paired with ~CMyD3DApplication()
//       Member variables should be initialized to a known state here.
//       The application window has not yet been created and no Direct3D device
//       has been created, so any initialization that depends on a window or
//       Direct3D should be deferred to a later stage.
//-----------------------------------------------------------------------------
CMyApplication::CMyApplication()
{
	m_lpszClassName = "PROGRAMEX_PUZZLE";
	m_lpszWindowName = "ProgrameX Puzzle";
	m_dwWidth = SCRWIDTH;
	m_dwHeight = SCRHEIGHT;

	m_bShowFrameRate = false;
	m_pScreen = NULL;

	SetCurrentDir();

	g_pGfx = new CGfx;
}

//-----------------------------------------------------------------------------
// Name: ~CMyD3DApplication()
// Desc: Application destructor.  Paired with CMyD3DApplication()
//-----------------------------------------------------------------------------
CMyApplication::~CMyApplication()
{
	if (g_pGfx)
	{
		delete g_pGfx;
		g_pGfx = NULL;
	}
}

//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Paired with FinalCleanup().
//       The window has been created and the IDirect3D9 interface has been
//       created, but the device has not been created yet.  Here you can
//       perform application-related initialization and cleanup that does
//       not depend on a device.
//-----------------------------------------------------------------------------
void CMyApplication::OneTimeSceneInit()
{
	// TODO: perform one time initialization

}

//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the display device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
void CMyApplication::ConfirmDevice( D3DCAPS9*, DWORD, D3DFORMAT )
{

}

//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Paired with DeleteDeviceObjects()
//       The device has been created.  Resources that are not lost on
//       Reset() can be created here -- resources in D3DPOOL_MANAGED,
//       D3DPOOL_SCRATCH, or D3DPOOL_SYSTEMMEM.  Image surfaces created via
//       CreateImageSurface are never lost and can be created here.  Vertex
//       shaders and pixel shaders can also be created here as they are not
//       lost on Reset().
//-----------------------------------------------------------------------------
void CMyApplication::InitDeviceObjects()
{
	// Globals
	g_pDevice = m_pDevice;

	POINT pt;
	if (GetCursorPos(&pt))
		if (ScreenToClient(m_hWnd, &pt))
		{
			g_iCursorX = pt.x;
			g_iCursorY = pt.y;
		}

	// Gfx
	if (g_pGfx)
		g_pGfx->InitDeviceObjects();

	// Screen
	m_pScreen = new CIntroScreen(m_hWnd, m_pDevice);
	m_pScreen->InitDeviceObjects();
}

//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Paired with InvalidateDeviceObjects()
//       The device exists, but may have just been Reset().  Resources in
//       D3DPOOL_DEFAULT and any other device state that persists during
//       rendering should be set here.  Render states, matrices, textures,
//       etc., that don't change during rendering can be set once here to
//       avoid redundant state setting during Render() or FrameMove().
//-----------------------------------------------------------------------------
void CMyApplication::RestoreDeviceObjects()
{
	// TODO: setup render states
	m_pDevice->SetRenderState(D3DRS_LIGHTING, false);
	m_pDevice->SetRenderState(D3DRS_ZENABLE, false);
	m_pDevice->SetRenderState(D3DRS_SPECULARENABLE, false);
	m_pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	m_pDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
	m_pDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	m_pDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
//	m_pDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);

	if (g_pGfx)
		g_pGfx->RestoreDeviceObjects();

	// Screen
	if (m_pScreen)
		m_pScreen->RestoreDeviceObjects();

/*	D3DMATERIAL9 mtrl;
	ZeroMemory(&mtrl, sizeof(mtrl));
	mtrl.Ambient.a = 1.0f;
	mtrl.Ambient.r = 1.0f;
	mtrl.Ambient.g = 1.0f;
	mtrl.Ambient.b = 1.0f;
	mtrl.Diffuse.a = 1.0f;
	mtrl.Diffuse.r = 1.0f;
	mtrl.Diffuse.g = 1.0f;
	mtrl.Diffuse.b = 1.0f;
	m_pDevice->SetMaterial(&mtrl);

	// Set the world matrix
	D3DXMATRIX matIdentity;
	D3DXMatrixIdentity(&matIdentity);
	m_pDevice->SetTransform(D3DTS_WORLD, &matIdentity);

/*
    // Setup a material
    D3DMATERIAL9 mtrl;
	mtrl.Ambient
    D3DUtil_InitMaterial( mtrl, 1.0f, 0.0f, 0.0f );
    m_pd3dDevice->SetMaterial( &mtrl );

    // Set up the textures
    m_pDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
    m_pDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    m_pDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    m_pDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE );
    m_pDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
    m_pDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE );

    // Set miscellaneous render states
    m_pDevice->SetRenderState( D3DRS_DITHERENABLE,   FALSE );
    m_pDevice->SetRenderState( D3DRS_SPECULARENABLE, FALSE );
    m_pDevice->SetRenderState( D3DRS_ZENABLE,        TRUE );
    m_pDevice->SetRenderState( D3DRS_AMBIENT,        0x000F0F0F );

    // Set the world matrix
    D3DXMATRIX matIdentity;
    D3DXMatrixIdentity( &matIdentity );
    m_pDevice->SetTransform( D3DTS_WORLD,  &matIdentity );

    // Set up our view matrix. A view matrix can be defined given an eye point,
    // a point to lookat, and a direction for which way is up. Here, we set the
    // eye five units back along the z-axis and up three units, look at the
    // origin, and define "up" to be in the y-direction.
    D3DXMATRIX matView;
    D3DXVECTOR3 vFromPt   = D3DXVECTOR3( 0.0f, 0.0f, -5.0f );
    D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vUpVec    = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &matView, &vFromPt, &vLookatPt, &vUpVec );
    m_pDevice->SetTransform( D3DTS_VIEW, &matView );

    // Set the projection matrix
    D3DXMATRIX matProj;
    FLOAT fAspect = ((FLOAT)m_dwWidth) / m_dwHeight;
    D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, fAspect, 1.0f, 100.0f );
    m_pDevice->SetTransform( D3DTS_PROJECTION, &matProj );

    // Set up lighting states
    D3DLIGHT9 light;
    D3DUtil_InitLight( light, D3DLIGHT_DIRECTIONAL, -1.0f, -1.0f, 2.0f );
    m_pDevice->SetLight( 0, &light );
    m_pDevice->LightEnable( 0, TRUE );
*/
}

//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
void CMyApplication::FrameMove(DWORD dwCzas, DWORD dwDeltaCzas)
{
    // TODO: update world
	if (m_pScreen)
	{
		m_pScreen->FrameMove(dwCzas, dwDeltaCzas);

		switch (m_pScreen->GetState())
		{
		case SCREENSTATE_INTROEND:
			if (m_bDeviceObjectsRestored)
				m_pScreen->InvalidateDeviceObjects();
			if (m_bDeviceObjectsInited)
				m_pScreen->DeleteDeviceObjects();
			delete m_pScreen;

			m_pScreen = new CMenuScreen(m_hWnd, m_pDevice);
			if (m_bDeviceObjectsInited)
				m_pScreen->InitDeviceObjects();
			if (m_bDeviceObjectsRestored)
				m_pScreen->RestoreDeviceObjects();			
			break;

		case SCREENSTATE_MENUEND:
			{
				if (m_bDeviceObjectsRestored)
					m_pScreen->InvalidateDeviceObjects();
				if (m_bDeviceObjectsInited)
					m_pScreen->DeleteDeviceObjects();
				delete m_pScreen;

				CGameScreen* game;
				m_pScreen = game = new CGameScreen(m_hWnd, m_pDevice);
				game->SetRunInfo(RunInfo);
				if (m_bDeviceObjectsInited)
					m_pScreen->InitDeviceObjects();
				if (m_bDeviceObjectsRestored)
					m_pScreen->RestoreDeviceObjects();			
				break;
			}

		case SCREENSTATE_GAMEEND:
			Close();
			break;
		}
	}

/*
	// Update the world state according to user input
    D3DXMATRIX matWorld;
    D3DXMATRIX matRotY;
    D3DXMATRIX matRotX;

    D3DXMatrixRotationX( &matRotX, 0.0f );
    D3DXMatrixRotationY( &matRotY, 0.0f );

    D3DXMatrixMultiply( &matWorld, &matRotX, &matRotY );
    m_pDevice->SetTransform( D3DTS_WORLD, &matWorld );
*/
}

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
void CMyApplication::Render()
{
	HRESULT hr;

	if (m_pScreen)
	{
		DWORD dwBkgColor = m_pScreen->GetBkgColor();
		if (dwBkgColor)
			if (FAILED(hr = m_pDevice->Clear(0, NULL, D3DCLEAR_TARGET, dwBkgColor, 1.0f, 0)))
				throw CError(ERRTYPE_DIRECTX, "Nie mona wyczyci urzdzenia", __FILE__, __LINE__, hr);

		if (SUCCEEDED(m_pDevice->BeginScene()))
		{
			m_pScreen->Render();
			if (m_bShowFrameRate)
				RenderFrameRate();
			m_pDevice->EndScene();
		}
	}
}

//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Overrrides the main WndProc, so the sample can do custom message
//       handling (e.g. processing mouse, keyboard, or menu commands).
//-----------------------------------------------------------------------------
LRESULT CMyApplication::MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch (msg)
	{
	case WM_KEYDOWN:
		if (m_pScreen)
			m_pScreen->KeyDown(wParam);
		break;
	case WM_KEYUP:
		if (m_pScreen)
			m_pScreen->KeyUp(wParam);
		break;
	case WM_CHAR:
		if (m_pScreen)
			m_pScreen->Char(wParam);
		break;
	case WM_MOUSEMOVE:
		g_iCursorX = LOWORD(lParam);
		g_iCursorY = HIWORD(lParam);

		if (m_pScreen)
			m_pScreen->MouseMove(LOWORD(lParam), HIWORD(lParam));
		break;
	case WM_LBUTTONDOWN:
		if (m_pScreen)
			m_pScreen->MouseDown(LOWORD(lParam), HIWORD(lParam), BUTTON_LEFT);
		break;
	case WM_LBUTTONUP:
		if (m_pScreen)
			m_pScreen->MouseUp(LOWORD(lParam), HIWORD(lParam), BUTTON_LEFT);
		break;
	case WM_RBUTTONDOWN:
		if (m_pScreen)
			m_pScreen->MouseDown(LOWORD(lParam), HIWORD(lParam), BUTTON_RIGHT);
		break;
	case WM_RBUTTONUP:
		if (m_pScreen)
			m_pScreen->MouseUp(LOWORD(lParam), HIWORD(lParam), BUTTON_RIGHT);
		break;
	case WM_MBUTTONDOWN:
		if (m_pScreen)
			m_pScreen->MouseDown(LOWORD(lParam), HIWORD(lParam), BUTTON_MIDDLE);
		break;
	case WM_MBUTTONUP:
		if (m_pScreen)
			m_pScreen->MouseUp(LOWORD(lParam), HIWORD(lParam), BUTTON_MIDDLE);
		break;
	}

	return CApplication::MsgProc(hWnd, msg, wParam, lParam);
}

//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Invalidates device objects.  Paired with RestoreDeviceObjects()
//-----------------------------------------------------------------------------
void CMyApplication::InvalidateDeviceObjects()
{
    // TODO: Cleanup any objects created in RestoreDeviceObjects()
	if (g_pGfx)
		g_pGfx->InvalidateDeviceObjects();

	// Screen
	if (m_pScreen)
		m_pScreen->InvalidateDeviceObjects();
}

//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Paired with InitDeviceObjects()
//       Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.  
//-----------------------------------------------------------------------------
void CMyApplication::DeleteDeviceObjects()
{
    // TODO: Cleanup any objects created in InitDeviceObjects()

	// Screen
	if (m_pScreen)
	{
		m_pScreen->DeleteDeviceObjects();
		delete m_pScreen;
		m_pScreen = NULL;
	}

	if (g_pGfx)
		g_pGfx->DeleteDeviceObjects();
}

//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Paired with OneTimeSceneInit()
//       Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
void CMyApplication::FinalCleanup()
{
    // TODO: Perform any final cleanup needed

}

void CMyApplication::ParseParameter(char *lpszParameter)
{
	// Pokazywanie FrameRate (FPS)
	if ( (strcmp(lpszParameter, "-f") == 0) || (strcmp(lpszParameter, "--framerate") == 0) )
		m_bShowFrameRate = true;
	// Parametr do informacji o uruchamianym pliku
	else if (ParseRunInfoParameter(lpszParameter))
		;
	else
		CApplication::ParseParameter(lpszParameter);
}

void CMyApplication::RenderFrameRate()
{
	LPD3DXFONT Font = g_pGfx->GetFont();
	Font->Begin();
	RECT rect = {8, 8, m_dwWidth-16, m_dwHeight-16};
	char s[32];
	sprintf(s, "FPS: %u", m_dwFrameRate);
	Font->DrawTextA(s, -1, &rect, 0, 0xffffffff);
	Font->End();
}

bool CMyApplication::ParseRunInfoParameter(std::string sParameter)
{
	RUNINFO::RUNINFOTYPE Type;
	std::string s;
	
	if (sParameter.find("-n:") == 0)
	{
		Type = RUNINFO::RUNINFOTYPE_NEW;
		s = sParameter.substr(3);
	}
	else if (sParameter.find("--new:") == 0)
	{
		Type = RUNINFO::RUNINFOTYPE_NEW;
		s = sParameter.substr(6);
	}
	else if (sParameter.find("-o:") == 0)
	{
		Type = RUNINFO::RUNINFOTYPE_OPEN;
		s = sParameter.substr(3);
	}
	else if	(sParameter.find("--open:") == 0)
	{
		Type = RUNINFO::RUNINFOTYPE_OPEN;
		s = sParameter.substr(7);
	}
	else
		return false;

	// Jeli ju by
	if (RunInfo.Type != RUNINFO::RUNINFOTYPE_NONE)
	{
		RunInfo.Type = RUNINFO::RUNINFOTYPE_NONE;
		return false;
	}

	if (Type == RUNINFO::RUNINFOTYPE_NEW)
	{
		size_t index = s.find("?");
		if (index == std::string.npos)
			return false;
		RunInfo.FileName = s.substr(0, index);
		s = s.substr(index+1);
		if (s == "")
			return false;
		RunInfo.Count = atoi(s.c_str());
		if (RunInfo.Count < 2)
			return false;
	}
	else if (Type == RUNINFO::RUNINFOTYPE_OPEN)
		RunInfo.FileName = s;
	// Dla bezpieczestwa
	else
		return false;
/* DEBUG
	char str[MAX_PATH];
	sprintf(str, "FileName: %s\nCount: %i", RunInfo.FileName.c_str(), RunInfo.Count);
	MessageBox(NULL, str, "Debug", MB_OK);
*/
	if (RunInfo.FileName == "")
		return false;

	RunInfo.Type = Type;
	return true;
}

bool CMyApplication::Confirm()
{
	if (RunInfo.Type == RUNINFO::RUNINFOTYPE_NONE)
		WinExec("Puzzle.exe", SW_SHOWNORMAL);
	return (RunInfo.Type != RUNINFO::RUNINFOTYPE_NONE);
}

void CMyApplication::SetCurrentDir()
{
	const int C_SIZE = MAX_PATH;
	char c[C_SIZE];
	GetModuleFileName(NULL, c, C_SIZE);

	char Drv[C_SIZE], Path[C_SIZE], Name[C_SIZE], Ext[C_SIZE];
	_splitpath(c, Drv, Path, Name, Ext);
	
	strcpy(c, Drv);
	strcat(c, Path);

	SetCurrentDirectory(c);
}