////////////////////////////////////////////////////////////////
// Autor : Regedit
// Data  : 2003-06-16
// Opis  : Obiekt gry
////////////////////////////////////////////////////////////////

#include <windows.h>
#include <string>
#include <vector>
#include <d3dx9.h>
#include <il.h>
#include <ilu.h>
#include <zlib.h>
#include <arrays.h>
#include "Error.h"
#include "Gfx.h"
#include "Scr.h"
#include "game.h"

////////////////////////////////////////////////////////////////
// Stae
const DWORD c_ElementFilter = D3DTEXF_LINEAR;
const float c_fScrollSpeed = 1.0f; // Prdko przewijania [px/ms]

const int BUTTON_IMAGE_WIDTH = 512;
const int BUTTON_IMAGE_HEIGHT = 512;
const int BUTTON_WIDTH_ON_IMAGE = 100;
const int BUTTON_HEIGHT_ON_IMAGE = 50;

const float BUTTON_WIDTH = 100.0f;
const float BUTTON_HEIGHT = 50.0f;
const float BUTTON_SPACE = 10.0f;
const float BUTTON_FIRST_X = 250.0f;
const float BUTTON_Y = 5.0f;

const float cPasek_Height = 60.0f;
const float cPasek_WysuwanieSpeed = 0.2f;

const int MESSAGE_TYPE_EXIT = 1;
const int MESSAGE_TYPE_ERROR = 2;
const int MESSAGE_TYPE_SAVE = 3;

const float MESSAGE_WIDTH = 400.0f;
const float MESSAGE_HEIGHT = 150.0f;
const float MESSAGE_SPACE_Y = 20.0f;
const float MESSAGE_BUTTON_SPACE_X = 10.0f;
const float MESSAGE_TEXT_MARGIN = 10.0f;

const char* MESSAGE_TEXT_EXIT = "Czy na pewno chcesz wyj?";

const int THUMBNAIL_WIDTH  = 512; // Szeroko tekstury miniaturki
const int THUMBNAIL_HEIGHT = 512; // Wysoko  tekstury miniaturki
const DWORD THUMBNAIL_BACKGROUND_COLOR = 0xffffffff; // Kolor ta tekstury miniatury

const char* HINT[PASEK_BUTTON_COUNT] = {
	"Przesuwa puzzle ze spodu na wierzch [Shift]",
	"Pokazuje puzzle pprzezroczyste tak, e wida to co jest pod spodem [Ctrl]",
	"Pokazuje miniaturk caego ukadanego obrazka [Tab]",
	"Zapisuje gr w celu jej pniejszego wczytania i kontynuowania",
	"Pozwala wyczy gr [ESC]"
};
const DWORD HINT_COLOR = 0xffffffff;

const int WIN_CZAS = 3000;

const int c_iSnapRange = 8; // Promie w pikselach, w jakim dziaa przyciganie

////////////////////////////////////////////////////////////////
// Profiler

class CProfiler
{
private:
	DWORD m_dwLastCzas;
	HANDLE m_hFile;
public:
	CProfiler();
	~CProfiler();
	void Write(char* lpszMessage);
};

CProfiler::CProfiler()
{
	m_dwLastCzas = GetTickCount();
	m_hFile = CreateFile("Profile.log", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
		FILE_ATTRIBUTE_NORMAL, NULL);
}

CProfiler::~CProfiler()
{
	if (m_hFile != INVALID_HANDLE_VALUE)
		CloseHandle(m_hFile);
}

void CProfiler::Write(char* lpszMessage)
{
	if (m_hFile != INVALID_HANDLE_VALUE)
	{
		DWORD dupa;
		DWORD dwCzas = GetTickCount();
		char* s = new char[strlen(lpszMessage)+32];
		sprintf(s, "%u - %s\r\n", dwCzas-m_dwLastCzas, lpszMessage);
		WriteFile(m_hFile, s, static_cast<DWORD>(strlen(s)), &dupa, NULL);
		delete [] s;
		m_dwLastCzas = dwCzas;
	}
}

////////////////////////////////////////////////////////////////
// CButton

CButton::CButton() :
	Over(false),
	Down(false),
	Clicked(false)
{
}

CButton::CButton(FLOATRECT aRect, int aImageIndex) :
	Rect(aRect),
	ImageIndex(aImageIndex),
	Over(false),
	Down(false),
	Clicked(false)
{
}

void CButton::MouseMove(WORD x, WORD y)
{
	Over = Rect.PointInRect( static_cast<float>(x), static_cast<float>(y) );
}

void CButton::MouseDown(WORD x, WORD y, SCREENMOUSEBUTTON button)
{
	if ( Rect.PointInRect( static_cast<float>(x), static_cast<float>(y) ) )
		Down = true;
}

void CButton::MouseUp(WORD x, WORD y, SCREENMOUSEBUTTON button)
{
	if (Down)
	{
		Down = false;
		if ( Rect.PointInRect( static_cast<float>(x), static_cast<float>(y) ) )
			Clicked = true;
	}
}

void CButton::Render()
{
	MYVERTEX v[4];
	int VertIndex = Down ? 1 : (Over ? 2 : 0);

	v[0].color = v[1].color = v[2].color = v[3].color = 0xffffffff;
	v[0].pos.z = v[1].pos.z = v[2].pos.z = v[3].pos.z = 0.0f;
	v[0].rhw = v[1].rhw = v[2].rhw = v[3].rhw = 1.0f;
	v[0].pos.x = v[1].pos.x = Rect.left;
	v[2].pos.x = v[3].pos.x = Rect.right;
	v[0].pos.y = v[2].pos.y = Rect.top;
	v[1].pos.y = v[3].pos.y = Rect.bottom;
	v[0].tx = v[1].tx = static_cast<float>( ImageIndex%4 * BUTTON_WIDTH_ON_IMAGE + 1) / static_cast<float>(BUTTON_IMAGE_WIDTH);
	v[2].tx = v[3].tx = static_cast<float>( (ImageIndex%4 + 1) * BUTTON_WIDTH_ON_IMAGE ) / static_cast<float>(BUTTON_IMAGE_WIDTH);
	v[0].ty = v[2].ty = static_cast<float>( (ImageIndex/4*3 + VertIndex) * BUTTON_HEIGHT_ON_IMAGE + 1) / static_cast<float>(BUTTON_IMAGE_HEIGHT);
	v[1].ty = v[3].ty = static_cast<float>( (ImageIndex/4*3 + (VertIndex+1)) * BUTTON_HEIGHT_ON_IMAGE ) / static_cast<float>(BUTTON_IMAGE_HEIGHT);

	g_pDevice->SetTexture(0, g_pGfx->GetTexture_Buttons());
	g_pDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
	g_pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_NONE);
	g_pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_NONE);

	g_pDevice->SetFVF(FVF_MYVERTEX);
	g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, v, sizeof(MYVERTEX));
}

bool CButton::GetClicked()
{
	bool b = Clicked;
	Clicked = false;
	return b;
}

////////////////////////////////////////////////////////////////
// CMessage

CMessage::CMessage(int aType, std::string aMessage)
{
	Type = aType;
	Message = aMessage;
	CButton* Button;

	switch (aType)
	{
	case MESSAGE_TYPE_EXIT:
		Button = new CButton;
		Button->SetImageIndex(5);
		Buttons.push_back(Button);
		Button = new CButton;
		Button->SetImageIndex(6);
		Buttons.push_back(Button);
		break;
	case MESSAGE_TYPE_ERROR:
		Button = new CButton;
		Button->SetImageIndex(7);
		Buttons.push_back(Button);
		break;
	case MESSAGE_TYPE_SAVE:
		Button = new CButton;
		Button->SetImageIndex(7);
		Buttons.push_back(Button);
		break;
	}
	SetButtonsPos();
}

CMessage::~CMessage()
{
	for (ButtonVector::iterator it = Buttons.begin(); it != Buttons.end(); it++)
		delete (*it);
}

void CMessage::RenderBackground()
{
	MYVERTEX v[4];

	v[0].color = v[1].color = v[2].color = v[3].color = 0xffffffff;
	v[0].pos.z = v[1].pos.z = v[2].pos.z = v[3].pos.z = 0.0f;
	v[0].rhw = v[1].rhw = v[2].rhw = v[3].rhw = 1.0f;
	v[0].pos.x = v[1].pos.x = static_cast<float>(SCRWIDTH) /2.0f - MESSAGE_WIDTH /2.0f;
	v[2].pos.x = v[3].pos.x = static_cast<float>(SCRWIDTH) /2.0f + MESSAGE_WIDTH /2.0f;
	v[0].pos.y = v[2].pos.y = static_cast<float>(SCRHEIGHT)/2.0f - MESSAGE_HEIGHT/2.0f;
	v[1].pos.y = v[3].pos.y = static_cast<float>(SCRHEIGHT)/2.0f + MESSAGE_HEIGHT/2.0f;

	g_pDevice->SetTexture(0, NULL);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_DIFFUSE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

	g_pDevice->SetFVF(FVF_MYVERTEX);
	g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, v, sizeof(MYVERTEX));
}

void CMessage::SetButtonsPos()
{
	float Width = BUTTON_WIDTH + MESSAGE_BUTTON_SPACE_X;
	float AllWidth = BUTTON_WIDTH * Buttons.size() +
		MESSAGE_BUTTON_SPACE_X * (Buttons.size()-1);

	for (DWORD i = 0; i < Buttons.size(); i++)
		Buttons[i]->SetRect(FLOATRECT(
			static_cast<float>(SCRWIDTH)/2.0f - AllWidth/2.0f + Width*i,
			static_cast<float>(SCRHEIGHT)/2.0f + MESSAGE_HEIGHT/2.0f - BUTTON_HEIGHT - MESSAGE_SPACE_Y,
			static_cast<float>(SCRWIDTH)/2.0f - AllWidth/2.0f + Width*i + BUTTON_WIDTH,
			static_cast<float>(SCRHEIGHT)/2.0f + MESSAGE_HEIGHT/2.0f - MESSAGE_SPACE_Y
			));
}

void CMessage::RenderMessage()
{
	RECT rect = {
		static_cast<int>( static_cast<float>(SCRWIDTH )/2.0f - MESSAGE_WIDTH /2.0f + MESSAGE_TEXT_MARGIN ),
		static_cast<int>( static_cast<float>(SCRHEIGHT)/2.0f - MESSAGE_HEIGHT/2.0f + MESSAGE_TEXT_MARGIN ),
		static_cast<int>( static_cast<float>(SCRWIDTH )/2.0f + MESSAGE_WIDTH /2.0f - MESSAGE_TEXT_MARGIN ),
		static_cast<int>( static_cast<float>(SCRHEIGHT)/2.0f + MESSAGE_HEIGHT/2.0f - BUTTON_HEIGHT - MESSAGE_SPACE_Y - MESSAGE_TEXT_MARGIN )
		};
	LPD3DXFONT font = g_pGfx->GetFont();

	font->Begin();
	font->DrawTextA(Message.c_str(), -1, &rect, DT_CENTER | DT_VCENTER | DT_WORDBREAK, 0xff000000);
	font->End();
}

void CMessage::Render()
{
	RenderBackground();
	RenderMessage();
	for (ButtonVector::iterator it = Buttons.begin(); it != Buttons.end(); it++)
		(*it)->Render();
}

void CMessage::MouseMove(WORD x, WORD y)
{
	for (ButtonVector::iterator it = Buttons.begin(); it != Buttons.end(); it++)
		(*it)->MouseMove(x, y);
}

void CMessage::MouseDown(WORD x, WORD y, SCREENMOUSEBUTTON button)
{
	for (ButtonVector::iterator it = Buttons.begin(); it != Buttons.end(); it++)
		(*it)->MouseDown(x, y, button);
}

void CMessage::MouseUp(WORD x, WORD y, SCREENMOUSEBUTTON button)
{
	for (ButtonVector::iterator it = Buttons.begin(); it != Buttons.end(); it++)
		(*it)->MouseUp(x, y, button);
}

////////////////////////////////////////////////////////////////
// CPasek

void CPasek::RenderBackground()
{
	MYVERTEX v[4];

	v[0].color = v[1].color = v[2].color = v[3].color = 0xffffffff;
	v[0].pos.z = v[1].pos.z = v[2].pos.z = v[3].pos.z = 0.0f;
	v[0].rhw = v[1].rhw = v[2].rhw = v[3].rhw = 1.0f;
	v[0].pos.x = v[1].pos.x = 0.0f;
	v[2].pos.x = v[3].pos.x = static_cast<float>(SCRWIDTH);
	v[0].pos.y = v[2].pos.y = static_cast<float>(SCRHEIGHT) - Height;
	v[1].pos.y = v[3].pos.y = static_cast<float>(SCRHEIGHT);

	g_pDevice->SetTexture(0, NULL);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_DIFFUSE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

	g_pDevice->SetFVF(FVF_MYVERTEX);
	g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, v, sizeof(MYVERTEX));
}

void CPasek::SetButtonsPos()
{
	for (int i = 0; i < PASEK_BUTTON_COUNT; i++)
		Buttons[i].SetRect(FLOATRECT(
			BUTTON_FIRST_X + i * (BUTTON_WIDTH + BUTTON_SPACE),
			static_cast<float>(SCRHEIGHT) - Height + BUTTON_Y,
			BUTTON_FIRST_X + i * (BUTTON_WIDTH + BUTTON_SPACE) + BUTTON_WIDTH,
			static_cast<float>(SCRHEIGHT) - Height + BUTTON_Y + BUTTON_HEIGHT
			));
}

CPasek::CPasek()
{
	Wysuniety = true;
	Height = cPasek_Height;

	for (int i = 0; i < PASEK_BUTTON_COUNT; i++)
		Buttons[i].SetImageIndex(i);
}

void CPasek::Render()
{
	if (Height > 0)
	{
		RenderBackground();
		RenderText();
		for (int i = 0; i < PASEK_BUTTON_COUNT; i++)
			Buttons[i].Render();
		RenderHint();
	}
}

void CPasek::FrameMove(DWORD dwCzas, DWORD dwDeltaCzas)
{
	if (Wysuniety && (Height < cPasek_Height))
		Height = min(cPasek_Height, Height + cPasek_WysuwanieSpeed*static_cast<float>(dwDeltaCzas));
	else if (!Wysuniety && (Height > 0.0f))
		Height = max(0.0f, Height - cPasek_WysuwanieSpeed*static_cast<float>(dwDeltaCzas));

	SetButtonsPos();
}

void CPasek::Wysun()
{
	Wysuniety = !Wysuniety;
}

void CPasek::MouseMove(WORD x, WORD y)
{
	for (int i = 0; i < PASEK_BUTTON_COUNT; i++)
		Buttons[i].MouseMove(x, y);

	// Hint
	Hint = "";
	for (int i = 0; i < PASEK_BUTTON_COUNT; i++)
		if (Buttons[i].GetOver())
		{
			Hint = HINT[i];
			break;
		}
}

FLOATRECT CPasek::GetRect()
{
	return FLOATRECT(
		0.0f,
		static_cast<float>(SCRHEIGHT)-Height,
		static_cast<float>(SCRWIDTH),
		static_cast<float>(SCRHEIGHT)
		);
}

bool CPasek::MouseDown(WORD x, WORD y, SCREENMOUSEBUTTON button)
{
	for (int i = 0; i < PASEK_BUTTON_COUNT; i++)
		Buttons[i].MouseDown(x, y, button);
	return GetRect().PointInRect(static_cast<float>(x), static_cast<float>(y));
}

bool CPasek::MouseUp(WORD x, WORD y, SCREENMOUSEBUTTON button)
{
	for (int i = 0; i < PASEK_BUTTON_COUNT; i++)
		Buttons[i].MouseUp(x, y, button);
	return GetRect().PointInRect(static_cast<float>(x), static_cast<float>(y));
}

////////////////////////////////////////////////////////////////
// GameScreen

CGameScreen::CGameScreen(HWND hWnd, LPDIRECT3DDEVICE9 pDevice) : CScreen(hWnd, pDevice)
{
	m_texPuzzle = NULL;
	m_texThumbnail = NULL;
	m_iWidth = 0;
	m_iHeight = 0;
	m_iElementCount = 0;
	m_iElementCountX = 0;
	m_iElementCountY = 0;
	m_iElementWidth = 0;
	m_iElementHeight = 0;
	m_iPuzzleWidth = 0;
	m_iPuzzleHeight = 0;
	m_iPuzzleA = 0;
	m_aMatrixX = NULL;
	m_aMatrixY = NULL;
	m_aMatrix = NULL;
	m_aWypustki = NULL;

	m_fScrollX = 0.0f;
	m_fScrollY = 0.0f;
	m_ArrowKeys.left = false;
	m_ArrowKeys.right = false;
	m_ArrowKeys.up = false;
	m_ArrowKeys.down = false;
	m_bAlphaMode1 = false;
	m_bAlphaMode2 = false;
	m_bThumbnailMode1 = false;
	m_bThumbnailMode2 = false;
	Czas = 0;

	m_bMoving = false;
	m_fDesktopWidth = 0;
	m_fDesktopHeight = 0;
	WinCzas = 0;
	bWin = false;

	Close = false;
	Message = NULL;
}

CGameScreen::~CGameScreen()
{
	if (Message)
		delete Message;
}

void CGameScreen::Load_InitializeDevIL(ILuint ImageNames[2])
{
	ilInit();
	iluInit();
	ilGenImages(2, ImageNames);
}

void CGameScreen::Load_FinalizeDevIL(ILuint ImageNames[2])
{
	ilDeleteImages(2, ImageNames);
	ilShutDown();
}

void CGameScreen::Load_LoadImage(ILuint ImageNames[2], const char* lpszFileName)
{
	ilBindImage(ImageNames[0]);

	// To musi by tak, bo w gupim DevILu typ ILString zdefiniowany jest
	//  jako char*, a nie const char*
	char* FileName = new char[strlen(lpszFileName)+1];
	strcpy(FileName, lpszFileName);
//	 ilEnable(IL_ORIGIN_SET); // DUPA! Nie dziaa!!!
	 ilLoadImage(FileName);
	delete [] FileName;

	if (ilGetError() != IL_NO_ERROR)
	{
		char s[MAX_PATH] = "Nie mona wczyta obrazka z pliku ";
		strcat(s, lpszFileName);
		throw CError(ERRTYPE_NORMAL, s, __FILE__, __LINE__);
	}

	// Odwrcenie obrazka
	char s1[MAX_PATH], s2[MAX_PATH], s3[MAX_PATH], s4[MAX_PATH];
	_splitpath(lpszFileName, s1, s2, s3, s4);
	for (DWORD i = 0; i < strlen(s4); i++)
		s4[i] = static_cast<char>(toupper(s4[i]));
	if ( (strcmp(s4, ".BMP") == 0) ||
		(strcmp(s4, ".SGI") == 0) ||
		(strcmp(s4, ".TIF") == 0) ||
		(strcmp(s4, ".TIFF") == 0)
		)
	{
//		Beep(1000, 100); * DEBUG
		iluFlipImage();
	}
}

void CGameScreen::Load_GetImageSize()
{
	m_iWidth  = ilGetInteger(IL_IMAGE_WIDTH );
	m_iHeight = ilGetInteger(IL_IMAGE_HEIGHT);

	if ( (m_iWidth <= MINELEMENTSIZE) || (m_iHeight <= MINELEMENTSIZE) )
		throw CError(ERRTYPE_NORMAL, "Obrazek jest za may", __FILE__, __LINE__, 0);
}

void CGameScreen::Load_GetPixels(ILuint ImageNames[2], array2d<IMAGECOLOR>* pPixels)
{
	ilBindImage(ImageNames[0]);
	ilCopyPixels(0, 0, 0, m_iWidth, m_iHeight, 1, IL_RGB, IL_UNSIGNED_BYTE, pPixels->GetPointer());
	if (ilGetError() != IL_NO_ERROR)
		throw CError(ERRTYPE_NORMAL, "Nie mona odczyta obrazka", __FILE__, __LINE__);
}

// Potgowanie cakowite 
inline int powi(int x, int y)
{
	return static_cast<int>( powf(static_cast<float>(x), static_cast<float>(y)) );
}

void CGameScreen::Load_Compute()
{
	for(;;)
	{
		// Liczba kolumn i wierszy
		m_iElementCountX = static_cast<int>( sqrtf(static_cast<float>(m_iElementCount) * static_cast<float>(m_iWidth) / static_cast<float>(m_iHeight))+0.5f );
		m_iElementCountY = static_cast<int>( sqrtf(static_cast<float>(m_iElementCount) * static_cast<float>(m_iHeight) / static_cast<float>(m_iWidth))+0.5f );

		// Szeroko i wysoko pojedynczego elementu
		m_iElementWidth = m_iWidth / m_iElementCountX;
		m_iElementHeight = m_iHeight / m_iElementCountY;

        // Jeli elementy za mae...
		if ((m_iElementWidth < MINELEMENTSIZE) || (m_iElementHeight < MINELEMENTSIZE))
			m_iElementCount -= m_iElementCount / 10 + 1;
		else
			break;
	}

	// Szeroko i wysoko obrazka raz jeszcze (by moe zosta lekko obcity)
	m_iWidth = m_iElementCountX * m_iElementWidth;
	m_iHeight = m_iElementCountY * m_iElementHeight;

	// Liczba elementw raz jeszcze (moga wyj mniejsza)
	m_iElementCount = m_iElementCountX * m_iElementCountY;

	// Potrzebna szeroko i wysoko tekstury z pocitymi puzzlami
	m_iPuzzleWidth = m_iWidth + (m_iElementWidth*(m_iElementCountX-1)) / 4;
	m_iPuzzleHeight = m_iHeight + (m_iElementHeight*(m_iElementCountY-1)) / 4;
		
	// A tutaj szukam dugoci A (bdcej potg dwjki) boku takiego kwadratu,
	// ktry pomieci w caoci obrazek
	for (int i = 0; i < 16; i++)
		if ( (powi(2, i) >= m_iPuzzleWidth) && (powi(2, i) >= m_iPuzzleHeight) )
		{
			m_iPuzzleA = powi(2, i);
			break;
		}

	// Szerko i wysoko pulpitu
	m_fDesktopWidth = m_iWidth * 1.5f + m_iElementCount * 2.0f;
	m_fDesktopHeight = m_iHeight * 1.5f + m_iElementCount * 2.0f;
}

void CGameScreen::Load_ComputeMatrices()
{
	int i;

	// MatrixX
	m_aMatrixX = new array1d<int>(m_iElementCountX);
	(*m_aMatrixX)[0] = 0;
	for (i = 1; i < m_iElementCountX; i++)
		(*m_aMatrixX)[i] = i*m_iElementWidth + rand()%(m_iElementWidth/4)-(m_iElementWidth/8);

	// MatrixY
	m_aMatrixY = new array1d<int>(m_iElementCountY);
	(*m_aMatrixY)[0] = 0;
	for (i = 1; i < m_iElementCountY; i++)
		(*m_aMatrixY)[i] = i*m_iElementHeight + rand()%(m_iElementHeight/4)-(m_iElementHeight/8);
}

void CGameScreen::Load_FillWypustki()
{
	m_aWypustki = new array2d<BOOLPAIR>(m_iElementCountX, m_iElementCountY);

	int x, y;
	for (y = 0; y < m_iElementCountY; y++)
		for (x = 0; x < m_iElementCountX; x++)
		{
			(*m_aWypustki)(x, y).b1 = (rand()%2 == 0);
			(*m_aWypustki)(x, y).b2 = (rand()%2 == 0);
		}
}

inline float CGameScreen::Load_Odleglosc(float x1, float y1, float x2, float y2)
{
	float f1 = x2 - x1;
	float f2 = y2 - y1;
	return sqrtf(f1*f1 + f2*f2);
}

void CGameScreen::Load_FillMatrix()
{
	m_aMatrix = new array2d<WORD>(m_iWidth, m_iHeight);

	int ex, ey; // Zmienne sterujce plami for interujcymi po kolejnych elementach
	int x, y; // Zmienen sterujce ptlami for interujcymi po kolejnych pikselach elementw
	int endx, endy; // Grny limit dla ptli for interujcych po kolejnych pikselach (x < endx), (y < endy)
	WORD value = 0; // Indeks elementu

	// Wypenienie Matrix prostoktami zgodnie z MatrixX i MatrixY
	for (ey = 0; ey < m_iElementCountY; ey++)
		for (ex = 0; ex < m_iElementCountX; ex++)
		{
			// * Dla kadego elementu * //
			endy = (ey == m_iElementCountY-1) ? m_iHeight : (*m_aMatrixY)[ey+1];
			endx = (ex == m_iElementCountX-1) ? m_iWidth  : (*m_aMatrixX)[ex+1];
			value = static_cast<WORD>( ey*m_iElementCountX + ex );

			// Wypenienie pikseli elementu
			for (y = (*m_aMatrixY)[ey]; y < endy; y++)
				for (x = (*m_aMatrixX)[ex]; x < endx; x++)
					(*m_aMatrix)(x, y) = value;
		}

	// Wypustki
	int i; // Licznik kierunku
		// Od 0 (w prawo) do 1 (w d)
	float a, dwa_a; // Promie elipsy i podwojony promie elipsy
		// (a <= min(m_iElementWidth, m_iElementHeight)/4)
	float d; // Poowa odlegoci midzy rodkami elipsy
		// (Odlego kadego rodka elipsy od punktu centralnego elipsy)
	float tmpx, tmpy; // Jeden z nich uywany jest zawsze do oznaczenia pozycji punkty centralnego elipsy,
		// ktra stanowi punkt wyjciowy do obliczenia jej rodkw z uwzgldnieniem d
	float f1x, f1y, f2x, f2y; // Pozycja rodkw elipsy
	WORD index; // Indeks elementu, ktrym ma zosta wypeniona elipsa
	RECT rect; // Prostokt ograliczajcy elips,
		// w obrbie ktrego sprawdzane s wszystkie piksele
	float fx, fy; // x i y przekonwertowane na float

	for (ey = 0; ey < m_iElementCountY; ey++)
		for (ex = 0; ex < m_iElementCountX; ex++)
			// * Dla kadego elementu * //
			for (i = 0; i < 2; i++)
			{
				// * Dla kierunku - w prawo lub w d * //
				/*
					i == 0	w prawo
					i == 1	w d
				*/

				// Ostatnia kolumna nie ma wypusek na prawo
				if ( (i == 0) && (ex == m_iElementCountX-1) )
					continue;
				// Ostatni wiersz nie ma wypustek na d
				if ( (i == 1) && (ey == m_iElementCountY-1) )
					continue;

				a = static_cast<float>( min(m_iElementWidth, m_iElementHeight) ) /
					( 6.0f + 1.0f * ( static_cast<float>(rand()) / RAND_MAX ) );
				dwa_a = a * 2.0f;
				d = 3.0f + static_cast<float>(rand()) / RAND_MAX * 4.0f;

				// W prawo
				if (i == 0)
				{
					tmpy = ((*m_aMatrixY)[ey]+( (ey==m_iElementCountY-1)?m_iHeight-1:(*m_aMatrixY)[ey+1] ))/2.0f + rand()/static_cast<float>(RAND_MAX)*m_iElementHeight/8.0f*(rand()%2==0?+1.0f:-1.0f);

					f1y = tmpy - d;
					f2y = tmpy + d;

					rect.top = static_cast<long>(f1y-a);
					rect.bottom = static_cast<long>(f2y+a);

					if ((*m_aWypustki)(ex, ey).b1)
					{
						// Wypustka
						f1x = (*m_aMatrixX)[ex+1] + a/2.5f;
						f2x = (*m_aMatrixX)[ex+1] + a/2.5f;

						rect.left = (*m_aMatrixX)[ex+1];
						rect.right = rect.left + static_cast<long>(dwa_a);

						index = ey*m_iElementCountX + ex;
					}
					else
					{
						// Wpustka
						f1x = (*m_aMatrixX)[ex+1] - a/2.5f;
						f2x = (*m_aMatrixX)[ex+1] - a/2.5f;

						rect.right = (*m_aMatrixX)[ex+1] ;
						rect.left = rect.right - static_cast<long>(dwa_a);

						index = ey*m_iElementCountX + ex + 1;
					}
				}
				// W d
				else
				{
					tmpx = ((*m_aMatrixX)[ex]+( (ex==m_iElementCountX-1)?m_iWidth-1:(*m_aMatrixX)[ex+1] ))/2.0f + rand()/static_cast<float>(RAND_MAX)*m_iElementWidth/8.0f*(rand()%2==0?+1.0f:-1.0f);

					f1x = tmpx - d;
					f2x = tmpx + d;

					rect.left = static_cast<long>(f1x-a);
					rect.right = static_cast<long>(f2x+a);

					if ((*m_aWypustki)(ex, ey).b2)
					{
						// Wypustka
						f1y = (*m_aMatrixY)[ey+1] + a/2.5f;
						f2y = (*m_aMatrixY)[ey+1] + a/2.5f;

						rect.top = (*m_aMatrixY)[ey+1];
						rect.bottom = rect.top + static_cast<long>(dwa_a);

						index = ey*m_iElementCountX + ex;
					}
					else
					{
						// Wpustka
						f1y = (*m_aMatrixY)[ey+1] - a/2.5f;
						f2y = (*m_aMatrixY)[ey+1] - a/2.5f;

						rect.bottom = (*m_aMatrixY)[ey+1] ;
						rect.top = rect.bottom - static_cast<long>(dwa_a);

						index = (ey+1)*m_iElementCountX + ex;
					}
				}

				// Narysowanie elipsy
				for (y = rect.top; y <= rect.bottom; y++)
					for (x = rect.left; x <= rect.right; x++)
					{
						fx = static_cast<float>(x);
						fy = static_cast<float>(y);
						if (Load_Odleglosc(fx, fy, f1x, f1y) + Load_Odleglosc(fx, fy, f2x, f2y) <= dwa_a)
							(*m_aMatrix)(x, y) = index;
					}
			}
}

void CGameScreen::Load_CreateTexture()
{
	HRESULT hr;
	if (FAILED(hr = g_pDevice->CreateTexture(m_iPuzzleA, m_iPuzzleA,
		1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED,&m_texPuzzle, NULL)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona utworzy tekstury puzzli", __FILE__, __LINE__, hr);
}

inline bool CGameScreen::Load_Belong(int x, int y, WORD index)
{
	if ( (x < 0) || (y < 0) || (x >= m_iWidth) || (y >= m_iHeight) )
		return false;

	return ( (*m_aMatrix)(x, y) == index );
}

inline DWORD CGameScreen::Load_GetColor(array2d<IMAGECOLOR>* pPixels, int x, int y, int light)
{
	IMAGECOLOR color = (*pPixels)(x, y);
	light *= 4;

	return D3DCOLOR_XRGB(
		max(0, min(255, light + color.r )),
		max(0, min(255, light + color.g )),
		max(0, min(255, light + color.b ))
		);
}

void CGameScreen::Load_FillTexture(array2d<IMAGECOLOR>* pPixels)
{
	HRESULT hr;
	D3DLOCKED_RECT lr;

	if (FAILED(hr = m_texPuzzle->LockRect(0, &lr, NULL, D3DLOCK_DISCARD)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona zapisywa do tekstuy puzzli", __FILE__, __LINE__, hr);

	int ex, ey; // Liczniki do iteracji po kolejnych elementach

	int x, y; // Liczniki do iteracji po kolejnych pikselach oryginalnego obrazka
		// Do bezporedniego indeksowania Pixels i Matrix
	int startx, starty; // Wartoci pocztkowe x i y
		// (x >= startx), (y >= starty)
	int endx, endy; // Wartoci kocowe licznikw x i y
		// (x < endx), (y < endy)

	int ix, iy; // Liczniki wewntrzne do cieniowania
	int startix, startiy; // Wartoci pocztkowe ix i iy
		// (ix >= startx), (iy >= starty)
	int endix, endiy; // Wartoci kocowe licznikw ix i iy
		// (x <= endx), (y <= endy)

	DWORD *Data; // Wskanik na pocztek wiersza tekstury
	WORD index; // Indeks elementu (???)
	int offsetx; // Warto dodawana do pozycji x, staa dla jednego elementu
	int r = 3; // Promie sprawdzania przynalenoci piksela do cieniowania

	// Wyzerowanie tekstury
	for (y = 0; y < m_iPuzzleA; y++)
		ZeroMemory(reinterpret_cast<DWORD*>(static_cast<BYTE*>(lr.pBits)+y*lr.Pitch), m_iPuzzleA*sizeof(DWORD));

	// Wypenienie tekstury
	for (ey = 0; ey < m_iElementCountY; ey++)
		for (ex = 0; ex < m_iElementCountX; ex++)
		{
			// * Dla kadego elementu * //
			starty = max(0, (*m_aMatrixY)[ey]-m_iElementHeight/4 );
			startx = max(0, (*m_aMatrixX)[ex]-m_iElementWidth/4 );

			endy = (ey == m_iElementCountY-1) ? m_iHeight : min(m_iHeight, (*m_aMatrixY)[ey+1]+m_iElementHeight/4);
			endx = (ex == m_iElementCountX-1) ? m_iWidth  : min(m_iWidth,  (*m_aMatrixX)[ex+1]+m_iElementWidth/4);

			offsetx = m_iElementWidth / 4 * ex;

			for (y = starty; y < endy; y++)
			{
				// * Dla kadej kolumny elementu * //
				Data = reinterpret_cast<DWORD*>( static_cast<BYTE*>(lr.pBits) + ( y+m_iElementHeight/4*ey )*lr.Pitch );

				for (x = startx; x < endx; x++)
				{
					// * Dla kadego piksela elementu * //
					index = ey*m_iElementCountX + ex;

					if ( (*m_aMatrix)(x, y) == index )
					{
						int iLight = 0;

						startiy = y-r;
						endiy = y+r;
						startix = x-r;
						endix = x+r;

						for (iy = startiy; iy <= endiy; iy++)
							for (ix = startix; ix <= endix; ix++)
								// * Dla kadego piksela ssiadujcego z tym w promieniu r * //
								if ( !Load_Belong(ix, iy, index) )
								{
									if ( (ix-x < 0) || (iy-y < 0) )
										iLight++;
									else
										iLight--;
								}

						Data[x+offsetx] = Load_GetColor(pPixels, x, y, iLight);
					}
				}
			}
		}

	m_texPuzzle->UnlockRect(0);
}

void CGameScreen::InitDeviceObjects()
{
	if (RunInfo.Type == RUNINFO::RUNINFOTYPE_NEW)
	{
		ILuint ImageNames[2];
		Load_InitializeDevIL(ImageNames);
		try
		{
			m_iElementCount = RunInfo.Count;
			Load_LoadImage(ImageNames, RunInfo.FileName.c_str());
			Load_GetImageSize();
			Load_CreateThumbnail(ImageNames);
			array2d<IMAGECOLOR> Pixels(m_iWidth, m_iHeight); // Tablica przechowujca wszystkie piksele obrazka
			Load_GetPixels(ImageNames, &Pixels);
			Load_Compute();
			Load_ComputeMatrices();
			Load_FillWypustki();
			Load_FillMatrix();
			Load_CreateTexture();
			Load_FillTexture(&Pixels);
			Load_CreateElements();

		}
		catch(...)
		{
			Load_FinalizeDevIL(ImageNames);
			Beep(1000, 500);
			throw;
		}
		Load_FinalizeDevIL(ImageNames);
	}
	else if (RunInfo.Type == RUNINFO::RUNINFOTYPE_OPEN)
	{
		Load2();
	}
	Pasek.SetElementCount(m_iElementCount);
}

void CGameScreen::RestoreDeviceObjects()
{
}

void CGameScreen::InvalidateDeviceObjects()
{
}

void CGameScreen::DeleteDeviceObjects()
{
	if (m_aWypustki) { delete m_aWypustki; m_aWypustki = NULL; }
	if (m_aMatrix) { delete m_aMatrix; m_aMatrix = NULL; }
	if (m_aMatrixX) { delete m_aMatrixX; m_aMatrixX = NULL; }
	if (m_aMatrixY) { delete m_aMatrixY; m_aMatrixY = NULL; }
	if (m_texThumbnail) { m_texThumbnail->Release(); m_texThumbnail = NULL; }
	if (m_texPuzzle) { m_texPuzzle->Release(); m_texPuzzle = NULL; }
}

void CGameScreen::Render()
{
	if (GetThumbnailMode())
		RenderThumbnail();
	else
	{
		// To - pulpit
		g_pGfx->RenderDesktop(m_fDesktopWidth, m_fDesktopHeight, m_fScrollX, m_fScrollY);

	/*	// Tekstura puzzli - tymczasowo!!!
		int iScrollX = static_cast<int>(m_fScrollX);
		int iScrollY = static_cast<int>(m_fScrollY);
		g_pGfx->DrawTexture(-iScrollX, -iScrollY, m_iPuzzleA-iScrollX, m_iPuzzleA-iScrollY, m_texPuzzle);
	*/
		// Element
		const float cShadow = -3.0f;

		for (GAMEELEMENTVECTOR::iterator it = m_vElements.begin();
			it != m_vElements.end(); it++)
		{
			if ( (it == --(m_vElements.end())) && m_bMoving )
			{
				RenderElement(*it, 0.0f, 0.0f, true);
				RenderElement(*it, cShadow, cShadow, false);
			}
			else
				RenderElement(*it, 0.0f, 0.0f, false);
		}
	}

	// Pasek
	Pasek.SetCzas(Czas);
	Pasek.Render();

	// Message
	if (Message)
		Message->Render();

	// Kursor
	g_pGfx->RenderCursor();
}

void CGameScreen::FrameMove(DWORD dwCzas, DWORD dwDeltaCzas)
{
	// Efekt gamma do wygrania
	if (WinCzas > 0)
	{
		if (dwCzas-WinCzas <= WIN_CZAS)
			DoWinEffect( static_cast<float>(dwCzas-WinCzas) / WIN_CZAS );
		else
		{
			DoWinEffectReset();
			WinCzas = 0;
		}
	}

	// Pasek
	Pasek.FrameMove(dwCzas, dwDeltaCzas);

	// 0: Shake
	if (Pasek.GetClicked(0))
		Shake();
	// 1: AlphaMode
	m_bAlphaMode2 = Pasek.GetDown(1);
	// 2: Podgld
	m_bThumbnailMode2 = Pasek.GetDown(2);
	// 3: Zapisz
	if (Pasek.GetClicked(3))
		Save();
	// 4: Wyjcie
	if (Pasek.GetClicked(4))
		AskForExit();

	// Jest komunikat
	if (Message)
	{
		switch (Message->GetType())
		{
		// Wyjcie
		case MESSAGE_TYPE_EXIT:
			// Tak
			if (Message->GetClicked(0))
				Close = true;
			// Nie
			else if (Message->GetClicked(1))
			{
				delete Message;
				Message = NULL;
			}
			break;
		case MESSAGE_TYPE_SAVE:
			// OK
			if (Message->GetClicked(0))
			{
				delete Message;
				Message = NULL;
			}
			break;
		}
	}

	// Nie ma komunikatu
	else
	{
		// Upyw czasu
		if (!bWin)
			Czas += min(dwDeltaCzas, 1000);

		// Przewijanie
		float vx = 0;
		float vy = 0;

		if (m_ArrowKeys.left || (g_iCursorX <= MOUSESCROLLLIMIT))
			vx -= c_fScrollSpeed;
		if (m_ArrowKeys.up || (g_iCursorY <= MOUSESCROLLLIMIT))
			vy -= c_fScrollSpeed;
		if (m_ArrowKeys.right || (g_iCursorX > 800-MOUSESCROLLLIMIT))
			vx += c_fScrollSpeed;
		if (m_ArrowKeys.down || (g_iCursorY > 600-MOUSESCROLLLIMIT))
			vy += c_fScrollSpeed;

		m_fScrollX += vx * dwDeltaCzas;
		m_fScrollY += vy * dwDeltaCzas;
		LimitScroll();

		// Przenoszenie
		if (m_bMoving)
		{
			GAMEELEMENT* e = &m_vElements[m_vElements.size()-1];
			e->x = max(0, min(m_fDesktopWidth - GetElementWidth(Index2EX(e->index)),
				g_iCursorX + m_fScrollX-m_fMoveX ));
			e->y = max(0, min(m_fDesktopHeight - GetElementHeight(Index2EY(e->index)),
				g_iCursorY + m_fScrollY-m_fMoveY ));
		}
	}
}

void CGameScreen::KeyDown(WPARAM wKey)
{
	if (Message)
		Message->KeyDown(wKey);
	else
	{
		switch (wKey)
		{
		case VK_LEFT:
			m_ArrowKeys.left = true;
			break;
		case VK_RIGHT:
			m_ArrowKeys.right = true;
			break;
		case VK_UP:
			m_ArrowKeys.up = true;
			break;
		case VK_DOWN:
			m_ArrowKeys.down = true;
			break;
		case VK_CONTROL:
			m_bAlphaMode1 = true;
			break;
		case VK_TAB:
			m_bThumbnailMode1 = true;
			break;
		case VK_SHIFT:
			Shake();
			break;
		case VK_SPACE:
			Pasek.Wysun();
			break;
		case VK_ESCAPE:
			AskForExit();
			break;
		}
	}
}

void CGameScreen::KeyUp(WPARAM wKey)
{
	if (Message)
		Message->KeyUp(wKey);
	else
	{
		switch (wKey)
		{
		case VK_LEFT:
			m_ArrowKeys.left = false;
			break;
		case VK_RIGHT:
			m_ArrowKeys.right = false;
			break;
		case VK_UP:
			m_ArrowKeys.up = false;
			break;
		case VK_DOWN:
			m_ArrowKeys.down = false;
			break;
		case VK_CONTROL:
			m_bAlphaMode1 = false;
			break;
		case VK_TAB:
			m_bThumbnailMode1 = false;
			break;
		}
	}
}

void CGameScreen::Char(WPARAM wKey)
{
}

void CGameScreen::MouseMove(WORD x, WORD y)
{
	if (Message)
		Message->MouseMove(x, y);
	else
	{
		Pasek.MouseMove(x, y);
	}
}

void CGameScreen::Element_MoveToBack(GAMEELEMENTVECTOR::iterator it)
{
	GAMEELEMENT element = *it;
	m_vElements.erase(it);
	m_vElements.push_back(element);
}

void CGameScreen::MouseDown(WORD x, WORD y, SCREENMOUSEBUTTON button)
{
	if (Message)
		Message->MouseDown(x, y, button);
	else if (GetThumbnailMode())
		Pasek.MouseDown(x, y, button);
	else
	{
		if (!Pasek.MouseDown(x, y, button))
		{
			if (button == BUTTON_LEFT)
			{
				if (!bWin)
				{
					if (!m_vElements.empty())
					{
						for (GAMEELEMENTVECTOR::iterator it = m_vElements.end()-1;
							it >= m_vElements.begin(); it--)
						{
							if (Collision(x+m_fScrollX, y+m_fScrollY, *it))
							{
								m_fMoveX = static_cast<float>(x) + m_fScrollX - it->x;
								m_fMoveY = static_cast<float>(y) + m_fScrollY - it->y;
								m_bMoving = true;

								Element_MoveToBack(it);

								break;
							}
						}
					}
				}
			}
		}
	}
}

void CGameScreen::MouseUp(WORD x, WORD y, SCREENMOUSEBUTTON button)
{
	if (Message)
		Message->MouseUp(x, y, button);
	else if (GetThumbnailMode())
		Pasek.MouseUp(x, y, button);
	else
	{
		if (!Pasek.MouseUp(x, y, button))
		{
			if (button == BUTTON_LEFT)
			{
				if (m_bMoving)
				{
					m_bMoving = false;
					int index = Snap();
					if (index == -1)
						m_vElements[m_vElements.size()-1].Ulozony = false;
					else
					{
						m_vElements[m_vElements.size()-1].Ulozony = true;
						m_vElements[index].Ulozony = true;
						Element_MoveToBegin(index);
						Element_FromToBack();
						if (Win())
						{
							bWin = true;
							DoWin();
						}
					}
				}
			}
		}
	}
}

int CGameScreen::GetState()
{
	if (Close)
		return SCREENSTATE_GAMEEND;
	else
		return SCREENSTATE_NONE;
}

void CGameScreen::Load_CreateElements()
{
	GAMEELEMENT element;

	for (int i = 0; i < m_iElementCount; i++)
	{
		element.x = static_cast<float>(rand()) * (m_fDesktopWidth -m_iElementWidth ) / RAND_MAX;
		element.y = static_cast<float>(rand()) * (m_fDesktopHeight-m_iElementHeight) / RAND_MAX;
		element.index = i;
		m_vElements.push_back(element);
	}
}

void CGameScreen::RenderElement(GAMEELEMENT element, float offsetx, float offsety, bool bShadow)
{
	int eix = element.index % m_iElementCountX;
	int eiy = element.index / m_iElementCountX;

	float x1 = element.x + offsetx;
	float y1 = element.y + offsety;

	float tx1 = static_cast<float>( (*m_aMatrixX)[eix] + m_iElementWidth /4*(eix) + 1 );
	float ty1 = static_cast<float>( (*m_aMatrixY)[eiy] + m_iElementHeight/4*(eiy) + 1 );

	float x2 = x1 + GetElementWidth(eix)  - 1;
	float y2 = y1 + GetElementHeight(eiy) - 1;

	float tx2 = tx1 + GetElementWidth(eix)  - 2.0f;
	float ty2 = ty1 + GetElementHeight(eiy) - 2.0f;

	if ( (eix > 0) && ((*m_aWypustki)(eix-1, eiy).b1 == false) )
	{
		x1 -= m_iElementWidth/4;
		tx1 -= m_iElementWidth/4;
	}
	if ( (eiy > 0) && ((*m_aWypustki)(eix, eiy-1).b2 == false) )
	{
		y1 -= m_iElementHeight/4;
		ty1 -= m_iElementHeight/4;
	}

	if ( (eix < m_iElementCountX-1) && ((*m_aWypustki)(eix, eiy).b1 == true) )
	{
		x2 += m_iElementWidth/4;
		tx2 += m_iElementWidth/4;
	}
	if ( (eiy < m_iElementCountY-1) && ((*m_aWypustki)(eix, eiy).b2 == true) )
	{
		y2 += m_iElementHeight/4;
		ty2 += m_iElementHeight/4;
	}


	MYVERTEX v1, v[4];

	v1.rhw = 1.0f;
	if (bShadow)
	{
		if (GetAlphaMode())
			v1.color = 0x00000000;
		else
			v1.color = 0x60000000;
	}
	else
	{
		if (GetAlphaMode())
			v1.color = 0x60ffffff;
		else
			v1.color = 0xffffffff;
	}
	v1.pos = D3DXVECTOR3(0.0f, 0.0f, 0.0f);

	v[0] = v1;
	v[0].pos.x = x1 - m_fScrollX;
	v[0].pos.y = y1 - m_fScrollY;
	v[0].tx = tx1 / m_iPuzzleA;
	v[0].ty = ty1 / m_iPuzzleA;

	v[1] = v1;
	v[1].pos.x = x1 - m_fScrollX;
	v[1].pos.y = y2 - m_fScrollY;
	v[1].tx = tx1 / m_iPuzzleA;
	v[1].ty = ty2 / m_iPuzzleA;

	v[2] = v1;
	v[2].pos.x = x2 - m_fScrollX;
	v[2].pos.y = y1 - m_fScrollY;
	v[2].tx = tx2 / m_iPuzzleA;
	v[2].ty = ty1 / m_iPuzzleA;

	v[3] = v1;
	v[3].pos.x = x2 - m_fScrollX;
	v[3].pos.y = y2 - m_fScrollY;
	v[3].tx = tx2 / m_iPuzzleA;
	v[3].ty = ty2 / m_iPuzzleA;

	g_pDevice->SetTexture(0, m_texPuzzle);
	g_pDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
	if (bShadow)
		g_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG2);
	else
		g_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);

	g_pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, c_ElementFilter);
	g_pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, c_ElementFilter);

	g_pDevice->SetFVF(FVF_MYVERTEX);
	g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, v, sizeof(MYVERTEX));
}

bool CGameScreen::Collision(float x, float y, GAMEELEMENT element)
{
	int xx = static_cast<int>(x - element.x) + (*m_aMatrixX)[Index2EX(element.index)];
	int yy = static_cast<int>(y - element.y) + (*m_aMatrixY)[Index2EY(element.index)];

	if ( (xx < 0) || (yy < 0) || (xx >= m_iWidth) || (yy >= m_iHeight) )
		return false;
	return ( (*m_aMatrix)(xx, yy) == element.index );
}

int CGameScreen::Snap()
{
	if (!m_vElements.empty())
	{
		float DeltaX;
		float DeltaY;

		// Element upuszczany
		GAMEELEMENT* e1 = &*(m_vElements.end()-1);
		int e1ex = Index2EX(e1->index);
		int e1ey = Index2EY(e1->index);

		// Kolejno sprawdzane elementy
		GAMEELEMENT* e2;
		int e2ex;
		int e2ey;
		int index;
		GAMEELEMENTVECTOR::iterator it;

		for (it = m_vElements.begin(), index = 0;
			it != m_vElements.end()-1; it++, index++)
		{
			e2 = &*it;

			e2ex = Index2EX(e2->index);
			e2ey = Index2EY(e2->index);

			DeltaX = e1->x - e2->x;
			DeltaY = e1->y - e2->y;

			if (e1ey == e2ey)
			{
				// Po prawej
				if (e1ex == e2ex+1)
					DeltaX -= GetElementWidth(e2ex);
				// Po lewej
				else if (e1ex == e2ex-1)
					DeltaX += GetElementWidth(e1ex);
				else
					continue;
			}
			else if (e1ex == e2ex)
			{
				// Na dole
				if (e1ey == e2ey+1)
					DeltaY -= GetElementHeight(e2ey);
				// U gry
				else if (e1ey == e2ey-1)
					DeltaY += GetElementHeight(e1ey);
				else
					continue;
			}
			else
				continue;

			if ( (DeltaX >= -c_iSnapRange) && (DeltaY >= -c_iSnapRange) &&
				(DeltaX <= c_iSnapRange) && (DeltaY <= c_iSnapRange) )
			{
				e1->x -= DeltaX;
				e1->y -= DeltaY;
				return index;
			}
		}
	}
	return -1;
}

void CGameScreen::Element_FromToBack()
{
	GAMEELEMENT element = m_vElements.back();
	m_vElements.pop_back();
	m_vElements.insert(m_vElements.begin(), element);
}

bool CPasek::GetClicked(int aIndex)
{
	return Buttons[aIndex].GetClicked();
}

bool CMessage::GetClicked(int aIndex)
{
	return Buttons[aIndex]->GetClicked();
}

void CGameScreen::Shake()
{
	DWORD i;
	GAMEELEMENTVECTOR vUlozone, vNieUlozone;
	GAMEELEMENT Przenoszony;

	// Przepisanie elementw z tablicy gwnej do tablic pomocniczych
	for (i = 0; i < m_vElements.size(); i++)
	{
		if (m_bMoving && (i == m_vElements.size()-1))
			Przenoszony = m_vElements[i];
		else
		{
			if (m_vElements[i].Ulozony)
				vUlozone.push_back(m_vElements[i]);
			else
				vNieUlozone.push_back(m_vElements[i]);
		}
	}
	
	// Wyczyszczenie tablicy gwnej
	m_vElements.clear();

	// Przepisanie do tablicy gwnej elementw uoonych
	for (i = 0; i < vUlozone.size(); i++)
		m_vElements.push_back(vUlozone[i]);

	// Odwrcenie kolejnoci tablicy nieuoonych
	GAMEELEMENT element;
	for (i = 0; i < vNieUlozone.size()/2; i++)
	{
		element = vNieUlozone[i];
		vNieUlozone[i] = vNieUlozone[vNieUlozone.size()-1-i];
		vNieUlozone[vNieUlozone.size()-1-i] = element;
	}

	// Przepisanie do tablicy gwnej elementw nieuoonych
	for (i = 0; i < vNieUlozone.size(); i++)
		m_vElements.push_back(vNieUlozone[i]);

	// Dodanie do tablicy gwnej elementu przenoszonego
	if (m_bMoving)
		m_vElements.push_back(Przenoszony);
}

void CGameScreen::Load_CreateThumbnail(ILuint ImageNames[2])
{
	// Bindujemy obrazek drugi, czyli miniaturk
	ilBindImage(ImageNames[1]);
	if (ilGetError() != IL_NO_ERROR)
		throw CError(ERRTYPE_NORMAL, "Bd podczas generowania podgldu", __FILE__, __LINE__);

	// Kopiujemy do niego oryginalny obrazek
	ilCopyImage(ImageNames[0]);
	if (ilGetError() != IL_NO_ERROR)
		throw CError(ERRTYPE_NORMAL, "Bd podczas generowania podgldu", __FILE__, __LINE__);

	// Ustawiamy filtrowanie liniowe
	iluImageParameter(ILU_FILTER, ILU_LINEAR);
	if (ilGetError() != IL_NO_ERROR)
		throw CError(ERRTYPE_NORMAL, "Bd podczas generowania podgldu", __FILE__, __LINE__);

	// === Skalowanie ===
	float Factor, Factor1, Factor2;

	// Jeli obrazek mieci si cay bez pomniejszania...
    if ((m_iWidth <= THUMBNAIL_WIDTH) && (m_iHeight <= THUMBNAIL_HEIGHT))
	{
		// ...to zachowuje swj naturalny rozmiar
		ThumbnailWidth  = m_iWidth ;
		ThumbnailHeight = m_iHeight;
	}
	// Jeli nie
	else
	{
		// Obliczenie mnonikw
		Factor1 = static_cast<float>(THUMBNAIL_WIDTH ) / static_cast<float>(m_iWidth );
		Factor2 = static_cast<float>(THUMBNAIL_HEIGHT) / static_cast<float>(m_iHeight);

		// Wybranie mniejszego mnonika
		Factor = (Factor1 < Factor2) ? Factor1 : Factor2;

		// No i dokonanie obliczenia docelowego rozmiaru
		ThumbnailWidth  = static_cast<int>( static_cast<float>(m_iWidth ) * Factor );
		ThumbnailHeight = static_cast<int>( static_cast<float>(m_iHeight) * Factor );
	}

	// Skalujemy miniaturk
	iluScale(ThumbnailWidth, ThumbnailHeight, 1);
	if (ilGetError() != IL_NO_ERROR)
		throw CError(ERRTYPE_NORMAL, "Bd podczas generowania podgldu", __FILE__, __LINE__);

	// === Teraz wypadaoby wycign z tego obrazka jego piksele
	array2d<IMAGECOLOR> Pixels(ThumbnailWidth, ThumbnailHeight); // Tablica przechowujca wszystkie piksele miniatury
	ilCopyPixels(0, 0, 0, ThumbnailWidth, ThumbnailHeight, 1, IL_RGB, IL_UNSIGNED_BYTE, Pixels.GetPointer());
	if (ilGetError() != IL_NO_ERROR)
		throw CError(ERRTYPE_NORMAL, "Nie mona odczyta miniatury", __FILE__, __LINE__);

	// === I tutaj zaczyna si kombinowanie z tekstur ===
	// Na pocztek tekstur trzeba, rzecz jasna, utworzy
	HRESULT hr;
	if (FAILED(hr = g_pDevice->CreateTexture(THUMBNAIL_WIDTH, THUMBNAIL_HEIGHT,
		1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED,&m_texThumbnail, NULL)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona utworzy tekstury miniaturki", __FILE__, __LINE__, hr);

	// Potem mona j zablokowa...
	D3DLOCKED_RECT lr;
	if (FAILED(hr = m_texThumbnail->LockRect(0, &lr, NULL, D3DLOCK_DISCARD)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona zapisywa do tekstury miniaturki", __FILE__, __LINE__, hr);

	// ...wypeni...
	int x, y;
	DWORD *Data;
	IMAGECOLOR c;
	int StartX = THUMBNAIL_WIDTH /2 - ThumbnailWidth /2;
	int StartY = THUMBNAIL_HEIGHT/2 - ThumbnailHeight/2;

	for (y = 0; y < THUMBNAIL_HEIGHT; y++)
	{
		Data = reinterpret_cast<DWORD*>( static_cast<BYTE*>(lr.pBits) + y*lr.Pitch );
		for (x = 0; x < THUMBNAIL_WIDTH; x++)
		{
			if ( (x >= StartX) && (y >= StartY) &&
				(x < StartX + ThumbnailWidth) && (y < StartY + ThumbnailHeight) )
			{
				c = Pixels(x-StartX, y-StartY);
				Data[x] = D3DCOLOR_XRGB(c.r, c.g, c.b);
			}
			else
				Data[x] = THUMBNAIL_BACKGROUND_COLOR;
		}
	}

	// ...i odblokowa
	m_texThumbnail->UnlockRect(0);
}

void CGameScreen::RenderThumbnail()
{
	MYVERTEX v[4];

	v[0].color = v[1].color = v[2].color = v[3].color = 0xffffffff;
	v[0].pos.z = v[1].pos.z = v[2].pos.z = v[3].pos.z = 0.0f;
	v[0].rhw = v[1].rhw = v[2].rhw = v[3].rhw = 1.0f;
	v[0].pos.x = v[1].pos.x = SCRWIDTH /2.0f - THUMBNAIL_WIDTH /2.0f;
	v[2].pos.x = v[3].pos.x = SCRWIDTH /2.0f + THUMBNAIL_WIDTH /2.0f;
	v[0].pos.y = v[2].pos.y = SCRHEIGHT/2.0f - THUMBNAIL_HEIGHT/2.0f;
	v[1].pos.y = v[3].pos.y = SCRHEIGHT/2.0f + THUMBNAIL_HEIGHT/2.0f;
	v[0].tx = v[1].tx = 0.0f;
	v[2].tx = v[3].tx = 1.0f;
	v[0].ty = v[2].ty = 0.0f;
	v[1].ty = v[3].ty = 1.0f;

	g_pDevice->SetTexture(0, m_texThumbnail);
	g_pDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
	g_pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	g_pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);

	g_pDevice->SetFVF(FVF_MYVERTEX);
	g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, v, sizeof(MYVERTEX));
}

void CGameScreen::Element_MoveToBegin(int index)
{
	GAMEELEMENT element = m_vElements[index];
	m_vElements.erase(m_vElements.begin()+index);
	m_vElements.insert(m_vElements.begin(), element);
}

void CPasek::RenderHint()
{
	if (!Hint.empty())
	{
		// To
		MYVERTEX v[4];

		v[0].color = v[1].color = v[2].color = v[3].color = 0x60000000;
		v[0].pos.z = v[1].pos.z = v[2].pos.z = v[3].pos.z = 0.0f;
		v[0].rhw = v[1].rhw = v[2].rhw = v[3].rhw = 1.0f;
		v[0].pos.x = v[1].pos.x = 0.0f;
		v[2].pos.x = v[3].pos.x = static_cast<float>(SCRWIDTH);
		v[0].pos.y = v[2].pos.y = static_cast<float>(SCRHEIGHT) - Height - 20.0f;
		v[1].pos.y = v[3].pos.y = static_cast<float>(SCRHEIGHT) - Height;

		g_pDevice->SetTexture(0, NULL);
		g_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
		g_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_DIFFUSE);
		g_pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

		g_pDevice->SetFVF(FVF_MYVERTEX);
		g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, v, sizeof(MYVERTEX));

		// Tekst
		LPD3DXFONT Font = g_pGfx->GetFont();
		RECT Rect = { 0, SCRHEIGHT-static_cast<int>(Height)-20,
			SCRWIDTH-1, SCRHEIGHT-static_cast<int>(Height) };

		Font->DrawTextA(Hint.c_str(), -1, &Rect, DT_VCENTER | DT_RIGHT | DT_SINGLELINE | DT_NOCLIP, HINT_COLOR);
	}
}

void CGameScreen::AskForExit()
{
	Message = new CMessage(MESSAGE_TYPE_EXIT, MESSAGE_TEXT_EXIT);
}

void CPasek::RenderText()
{
	__int64 iSec = Czas / ( 1000 ) % 60;
	__int64 iMin = Czas / ( 1000*60) % 60;
	__int64 iHour = Czas / ( 1000*60*60);

	char lpszSec[3], lpszMin[3], lpszHour[32];
	_i64toa(iSec, lpszSec, 10);
	_i64toa(iMin, lpszMin, 10);
	_i64toa(iHour, lpszHour, 10);

	std::string s;
	if (strlen(lpszHour) == 1)
		s += "0";
	s += lpszHour;
	s += ":";
	if (strlen(lpszMin) == 1)
		s += "0";
	s += lpszMin;
	s += ":";
	if (strlen(lpszSec) == 1)
		s += "0";
	s += lpszSec;

	// Nie wiem dlaczego to tak dziaa, ale grunt e dziaa
	//  (mowa jest o inicjalizacji wsprzdnych tego prostokta)
	RECT Rect = {
		static_cast<int>(cPasek_Height)/2,
		SCRHEIGHT-static_cast<int>(Height-(cPasek_Height-Height)),
		SCRWIDTH,
		SCRHEIGHT/*+static_cast<int>(cPasek_Height-Height)*/
	};

	////////////////////////////////////////////////////////////////

	char lpszElementCount[16];
	itoa(ElementCount, lpszElementCount, 10);

	RECT Rect2 = {
		175,
		SCRHEIGHT-static_cast<int>(Height-(cPasek_Height-Height)),
		SCRWIDTH,
		SCRHEIGHT
	};

	LPD3DXFONT font = g_pGfx->GetFont();
	font->Begin();
	font->DrawTextA(s.c_str(), -1, &Rect, DT_NOCLIP | DT_VCENTER, 0xff000000);
	font->DrawTextA(lpszElementCount, -1, &Rect2, DT_NOCLIP | DT_VCENTER, 0xff00007f);
	font->End();
}

bool CGameScreen::Win()
{
	int i; // Do ptli for
	GAMEELEMENT e; // Element tymczasowy (dla wygody zapisu)

	// Znalezienie pozycji pierwszego puzzla
	float FirstX; // Tutaj zostanie ta pozycja zapamitana
	float FirstY; // ^
	for (i = 0; i < m_iElementCount; i++)
	{
		e = m_vElements[i];
		if (e.index == 0)
		{
			FirstX = e.x;
			FirstY = e.y;
			break;
		}
	}

	// Sprawdzenie wszystkich puzzli
	for (i = 0; i < m_iElementCount; i++)
	{
		e = m_vElements[i];
		if (e.index != 0)
		{
			// Jeli ten puzel nie jest na waciwym miejscu wzgldem pierwszego...
			if ( ( fabsf( e.x - (FirstX + (*m_aMatrixX)[e.index%m_iElementCountX]) ) >= c_iSnapRange) ||
				( fabsf( e.y - (FirstY + (*m_aMatrixY)[e.index/m_iElementCountX]) ) >= c_iSnapRange)
				)
				// ...to nie ma wygranej
				return false;
		}
	}

	// A jeli kod doszed a tu, to znaszy e jest :)
	return true;
}

void CGameScreen::Save()
{
	try
	{
		// Otwarcie pliku
		char lpszFileName[MAX_PATH];
		SaveGetFileName(lpszFileName);
		gzFile file = gzopen(lpszFileName, "wb8");
		// Bd podczas otwierania
		if (file == NULL)
			throw CError(ERRTYPE_NORMAL, "Nie mona otworzy pliku aby zapisa gr", __FILE__, __LINE__);

		try
		{
			gzwrite(file, &m_iElementCount, sizeof(m_iElementCount));
			gzwrite(file, &m_iPuzzleA, sizeof(m_iPuzzleA));
			gzwrite(file, &m_iWidth, sizeof(m_iWidth));
			gzwrite(file, &m_iHeight, sizeof(m_iHeight));
			gzwrite(file, &m_iElementCountX, sizeof(m_iElementCountX));
			gzwrite(file, &m_iElementCountY, sizeof(m_iElementCountY));
			gzwrite(file, &m_iElementWidth, sizeof(m_iElementWidth));
			gzwrite(file, &m_iElementHeight, sizeof(m_iElementHeight));
			gzwrite(file, &m_iPuzzleWidth, sizeof(m_iPuzzleWidth));
			gzwrite(file, &m_iPuzzleHeight, sizeof(m_iPuzzleHeight));
			gzwrite(file, &m_fDesktopWidth, sizeof(m_fDesktopWidth));
			gzwrite(file, &m_fDesktopHeight, sizeof(m_fDesktopHeight));
			gzwrite(file, &Czas, sizeof(Czas));

			SaveMatrices(file);
			SaveWypustki(file);
			SaveMatrix(file);

			SaveThumbnail(file);
			SaveTexture(file);

			SaveElements(file);
		}
		catch(...)
		{
			gzclose(file);
			throw;
		}

		// Zamknicie pliku
		gzclose(file);

		Message = new CMessage(MESSAGE_TYPE_SAVE, "Gra zostaa zapisana");
	}
	catch(...)
	{
		Message = new CMessage(MESSAGE_TYPE_SAVE, "Bd podczas zapisywania gry");
	}
}

void CGameScreen::SaveMatrices(gzFile file)
{
	gzwrite(file, m_aMatrixX->GetPointer(), sizeof(int)*m_iElementCountX);
	gzwrite(file, m_aMatrixY->GetPointer(), sizeof(int)*m_iElementCountY);
}

void CGameScreen::SaveWypustki(gzFile file)
{
	gzwrite(file, m_aWypustki->GetPointer(), sizeof(BOOLPAIR)*m_iElementCountX*m_iElementCountY);
}

void CGameScreen::SaveMatrix(gzFile file)
{
	gzwrite(file, m_aMatrix->GetPointer(), sizeof(WORD)*m_iWidth*m_iHeight);
}

void CGameScreen::SaveThumbnail(gzFile file)
{
	// Zapisanie rozmiaru tekstury miniaturki
	gzwrite(file, &ThumbnailWidth, sizeof(ThumbnailWidth));
	gzwrite(file, &ThumbnailHeight, sizeof(ThumbnailHeight));

	// * ZAPISANIE TEKSTURY *
	// Zablokowanie tekstury
	HRESULT hr;
	D3DLOCKED_RECT lr;
	if (FAILED(hr = m_texThumbnail->LockRect(0, &lr, NULL, D3DLOCK_READONLY)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona odczyta do tekstury miniaturki", __FILE__, __LINE__, hr);

	// Zapisanie treci
	int y;
	DWORD *Data;

	for (y = 0; y < THUMBNAIL_HEIGHT; y++)
	{
		Data = reinterpret_cast<DWORD*>( static_cast<BYTE*>(lr.pBits) + y*lr.Pitch );
		gzwrite(file, Data, sizeof(DWORD)*THUMBNAIL_WIDTH);
	}

	// Obdlokowanie tekstury
	m_texThumbnail->UnlockRect(0);
}

void CGameScreen::SaveTexture(gzFile file)
{
	HRESULT hr;
	D3DLOCKED_RECT lr;
	if (FAILED(hr = m_texPuzzle->LockRect(0, &lr, NULL, D3DLOCK_READONLY)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona odczyta tekstuy puzzli", __FILE__, __LINE__, hr);

	DWORD* Data;
	for (int y = 0; y < m_iPuzzleA; y++)
	{
		Data = reinterpret_cast<DWORD*>( static_cast<BYTE*>(lr.pBits) + y*lr.Pitch );
		gzwrite(file, Data, sizeof(DWORD)*m_iPuzzleA);
	}

	m_texPuzzle->UnlockRect(0);   
}

void CGameScreen::SaveElements(gzFile file)
{
	GAMEELEMENT e;
	for (int i = 0; i < m_iElementCount; i++)
	{
		e = m_vElements[i];
		gzwrite(file, &e, sizeof(e));
	}
}

void CGameScreen::Load2()
{
	try
	{
		// Otwarcie pliku
		gzFile file = gzopen(RunInfo.FileName.c_str(), "rb");
		// Bd podczas otwierania
		if (file == NULL)
			throw CError(ERRTYPE_NORMAL, "Nie mona otworzy pliku z zapisan gr", __FILE__, __LINE__);

		try
		{
			gzread(file, &m_iElementCount, sizeof(m_iElementCount));
			gzread(file, &m_iPuzzleA, sizeof(m_iPuzzleA));
			gzread(file, &m_iWidth, sizeof(m_iWidth));
			gzread(file, &m_iHeight, sizeof(m_iHeight));
			gzread(file, &m_iElementCountX, sizeof(m_iElementCountX));
			gzread(file, &m_iElementCountY, sizeof(m_iElementCountY));
			gzread(file, &m_iElementWidth, sizeof(m_iElementWidth));
			gzread(file, &m_iElementHeight, sizeof(m_iElementHeight));
			gzread(file, &m_iPuzzleWidth, sizeof(m_iPuzzleWidth));
			gzread(file, &m_iPuzzleHeight, sizeof(m_iPuzzleHeight));
			gzread(file, &m_fDesktopWidth, sizeof(m_fDesktopWidth));
			gzread(file, &m_fDesktopHeight, sizeof(m_fDesktopHeight));
			gzread(file, &Czas, sizeof(Czas));

			Load2Matrices(file);
			Load2Wypustki(file);
			Load2Matrix(file);

			Load2Thumbnail(file);
			Load2Texture(file);

			Load2Elements(file);
		}
		catch(...)
		{
			gzclose(file);
			throw;
		}

		// Zamknicie pliku
		gzclose(file);
	}
	catch(...)
	{
		Beep(2000, 100);
		throw;
	}
}

void CGameScreen::Load2Matrices(gzFile file)
{
	m_aMatrixX = new array1d<int>(m_iElementCountX);
	m_aMatrixY = new array1d<int>(m_iElementCountY);

	gzread(file, m_aMatrixX->GetPointer(), sizeof(int)*m_iElementCountX);
	gzread(file, m_aMatrixY->GetPointer(), sizeof(int)*m_iElementCountY);
}

void CGameScreen::Load2Wypustki(gzFile file)
{
	m_aWypustki = new array2d<BOOLPAIR>(m_iElementCountX, m_iElementCountY);
	gzread(file, m_aWypustki->GetPointer(), sizeof(BOOLPAIR)*m_iElementCountX*m_iElementCountY);
}

void CGameScreen::Load2Matrix(gzFile file)
{
	m_aMatrix = new array2d<WORD>(m_iWidth, m_iHeight);
	gzread(file, m_aMatrix->GetPointer(), sizeof(WORD)*m_iWidth*m_iHeight);
}

void CGameScreen::Load2Thumbnail(gzFile file)
{
	// Odczytanie rozmiaru tekstury miniaturki
	gzread(file, &ThumbnailWidth, sizeof(ThumbnailWidth));
	gzread(file, &ThumbnailHeight, sizeof(ThumbnailHeight));

	// * ODCZYTANIE TEKSTURY *
	// Na pocztek tekstur trzeba, rzecz jasna, utworzy
	HRESULT hr;
	if (FAILED(hr = g_pDevice->CreateTexture(THUMBNAIL_WIDTH, THUMBNAIL_HEIGHT,
		1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED,&m_texThumbnail, NULL)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona utworzy tekstury miniaturki", __FILE__, __LINE__, hr);

	// Zablokowanie tekstury
	D3DLOCKED_RECT lr;
	if (FAILED(hr = m_texThumbnail->LockRect(0, &lr, NULL, D3DLOCK_DISCARD)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona odczyta tekstury miniaturki", __FILE__, __LINE__, hr);

	// Odczytanie treci
	int y;
	DWORD *Data;

	for (y = 0; y < THUMBNAIL_HEIGHT; y++)
	{
		Data = reinterpret_cast<DWORD*>( static_cast<BYTE*>(lr.pBits) + y*lr.Pitch );
		gzread(file, Data, sizeof(DWORD)*THUMBNAIL_WIDTH);
	}

	// Obdlokowanie tekstury
	m_texThumbnail->UnlockRect(0);
}

void CGameScreen::Load2Texture(gzFile file)
{
	HRESULT hr;
	if (FAILED(hr = g_pDevice->CreateTexture(m_iPuzzleA, m_iPuzzleA,
		1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &m_texPuzzle, NULL)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona utworzy tekstury puzzli", __FILE__, __LINE__, hr);

	D3DLOCKED_RECT lr;
	if (FAILED(hr = m_texPuzzle->LockRect(0, &lr, NULL, D3DLOCK_DISCARD)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona zapisa tekstuy puzzli", __FILE__, __LINE__, hr);

	DWORD* Data;
	for (int y = 0; y < m_iPuzzleA; y++)
	{
		Data = reinterpret_cast<DWORD*>( static_cast<BYTE*>(lr.pBits) + y*lr.Pitch );
		gzread(file, Data, sizeof(DWORD)*m_iPuzzleA);
	}

	m_texPuzzle->UnlockRect(0);
}

void CGameScreen::Load2Elements(gzFile file)
{
	GAMEELEMENT e;
	for (int i = 0; i < m_iElementCount; i++)
	{
		gzread(file, &e, sizeof(e));
		m_vElements.push_back(e);
	}
}

DWORD CGameScreen::GetBkgColor()
{
	return GetThumbnailMode() ? 0xffffffff : 0xff000000;
}

void CGameScreen::SaveGetFileName(char* lpszFileName)
{
	if (RunInfo.Type == RUNINFO::RUNINFOTYPE_OPEN)
		strcpy(lpszFileName, RunInfo.FileName.c_str());
	else if (RunInfo.Type == RUNINFO::RUNINFOTYPE_NEW)
	{
		char drive[MAX_PATH], dir[MAX_PATH], fname[MAX_PATH], ext[MAX_PATH];
		_splitpath(RunInfo.FileName.c_str(), drive, dir, fname, ext);
		strcat(fname, ext);
		sprintf(lpszFileName, "%s (%i).pzz", fname, m_iElementCount);
	}
}

void CMessage::KeyDown(WPARAM wKey)
{
	if ( (wKey == VK_RETURN) || (wKey == VK_SPACE) )
		Buttons[0]->SetClicked(true);
	else if ( (wKey == VK_ESCAPE) && (Type == MESSAGE_TYPE_EXIT) )
		Buttons[1]->SetClicked(true);
}

void CMessage::KeyUp(WPARAM wKey)
{
	// Pusto...
}

void CGameScreen::LimitScroll()
{
	if (m_fScrollX > m_fDesktopWidth-SCRWIDTH)
		m_fScrollX = m_fDesktopWidth-SCRWIDTH;
	if (m_fScrollX < 0.0f)
		m_fScrollX = 0.0f;

	if (m_fScrollY > m_fDesktopHeight-SCRHEIGHT+Pasek.GetHeight())
		m_fScrollY = m_fDesktopHeight-SCRHEIGHT+Pasek.GetHeight();
	if (m_fScrollY < 0.0f)
		m_fScrollY = 0.0f;
}

void CGameScreen::DoWin()
{
	WinCzas = GetTickCount();
}

void CGameScreen::DoWinEffect(float Progress)
{
	// Deklaracja
	int i;
	BYTE byte;
	D3DGAMMARAMP ramp;

	// Wypenienie
	for (i = 0; i < 256; i++)
	{
//		byte = static_cast<BYTE>( (1.0f-Progress) * (i) + (Progress) * (255-i));
		byte = i + static_cast<BYTE>( Progress * 256.0f );
		ramp.red[i] = byte;
		ramp.green[i] = byte;
		ramp.blue[i] = byte;
	}

	// Zabezpieczenie przed burakami
	for (i = 0; i < 256; i++)
	{
		ramp.red[i] = ramp.red[i] | (ramp.red[i] << 8);
		ramp.green[i] = ramp.green[i] | (ramp.green[i] << 8);
		ramp.blue[i] = ramp.blue[i] | (ramp.blue[i] << 8);
	}

	// Ustawienie
	g_pDevice->SetGammaRamp(0, D3DSGR_NO_CALIBRATION, &ramp);
}

void CGameScreen::DoWinEffectReset()
{
	// Deklaracja
	int i;
	D3DGAMMARAMP ramp;

	// Wypenienie
	for (i = 0; i < 256; i++)
	{
		ramp.red[i] = i;
		ramp.green[i] = i;
		ramp.blue[i] = i;
	}

	// Zabezpieczenie przed burakami
	for (i = 0; i < 256; i++)
	{
		ramp.red[i] = ramp.red[i] | (ramp.red[i] << 8);
		ramp.green[i] = ramp.green[i] | (ramp.green[i] << 8);
		ramp.blue[i] = ramp.blue[i] | (ramp.blue[i] << 8);
	}

	// Ustawienie
	g_pDevice->SetGammaRamp(0, D3DSGR_CALIBRATE, &ramp);
}