////////////////////////////////////////////////////////////////
// Autor : Regedit
// Data  : 2003-06-16
// Opis  : Obiekt gry
////////////////////////////////////////////////////////////////

#pragma once

#define MOUSESCROLLLIMIT 3 // Liczba pikseli kursora myszy od grawdzi ekranu potrzeba by ekran zacz si przewija
#define MINELEMENTSIZE 48 // Minimalna szeroko i wysoko pojedynczego elementu
#define PASEK_BUTTON_COUNT 5

// Stan wcinicia strzaek na klawiaturze
typedef struct _ARROWKEYS
{
	bool left;
	bool right;
	bool up;
	bool down;
} ARROWKEYS;

// 3-bajtowa struktura do przechowywania koloru piksela
typedef struct _IMAGECOLOR
{
	BYTE r;
	BYTE g;
	BYTE b;
} IMAGECOLOR;

// Para zmiennych logicznych do wypustek elementu
typedef struct _BOOLPAIR
{
	bool b1;
	bool b2;
	_BOOLPAIR(bool ab1, bool ab2) { b1 = ab1; b2 = ab2; }
	_BOOLPAIR() { }
} BOOLPAIR;

// Klasa reprezentuje pojedynczy element - "puzzel"
struct GAMEELEMENT
{
	float x, y; // Pozycja lewego grnego rogu
		// prostokta ograniczajcego waciwy prostokt elementu (bez wypustek)
	WORD index; // Indeks elementu
	bool Ulozony;
	GAMEELEMENT() : Ulozony(false) { }
};

typedef std::vector<GAMEELEMENT> GAMEELEMENTVECTOR;

typedef struct tagFLOATRECT
{
	float left;
	float top;
	float right;
	float bottom;

	tagFLOATRECT() { }
	tagFLOATRECT(tagFLOATRECT &aRect) { left = aRect.left; top = aRect.top; right = aRect.right; bottom = aRect.bottom; }
	tagFLOATRECT(float aLeft, float aTop, float aRight, float aBottom) { left = aLeft; top = aTop; right = aRight; bottom = aBottom; }
	float GetWidth() { return right-left; }
	float GetHeight() { return bottom-top; }
	float SetWidth(float width) { right = left + width; }
	float SetHeight(float height) { bottom = top + height; }
	bool PointInRect(float x, float y) { return (x >= left) && (x < right) && (y >= top) && (y < bottom); }
} FLOATRECT;

class CButton
{
private:
	FLOATRECT Rect;
	int ImageIndex;
	bool Over;
	bool Down;
	bool Clicked;
public:
	CButton();
	CButton(FLOATRECT aRect, int aImageIndex);
	void SetRect(FLOATRECT aRect) { Rect = aRect; }
	void SetImageIndex(int aImageIndex) { ImageIndex = aImageIndex; }
	void MouseMove(WORD x, WORD y);
	void MouseDown(WORD x, WORD y, SCREENMOUSEBUTTON button);
	void MouseUp(WORD x, WORD y, SCREENMOUSEBUTTON button);
	void Render();
	bool GetClicked(); // Odczytuje, a take zeruje pole Clicked
	void SetClicked(bool aClicked) { Clicked = aClicked; }
	bool GetDown() { return Down; }
	bool GetOver() { return Over; }
};

typedef std::vector<CButton*> ButtonVector;

class CMessage
{
private:
	int Type;
	std::string Message;
	ButtonVector Buttons;

	void RenderBackground();
	void RenderMessage();
	void SetButtonsPos();
public:
	CMessage(int aType, std::string aMessage);
	~CMessage();
	void SetMessage(std::string aMessage) { Message = aMessage; }
	void Render();
	void MouseMove(WORD x, WORD y);
	void MouseDown(WORD x, WORD y, SCREENMOUSEBUTTON button);
	void MouseUp(WORD x, WORD y, SCREENMOUSEBUTTON button);
	void KeyDown(WPARAM wKey);
	void KeyUp(WPARAM wKey);
	int GetType() { return Type; }
	bool GetClicked(int aIndex);
};

class CPasek
{
private:
	bool Wysuniety;
	float Height;
	CButton Buttons[PASEK_BUTTON_COUNT];
	std::string Hint;
	__int64 Czas;
	int ElementCount;

	void RenderBackground();
	void RenderHint();
	void RenderText();
	void SetButtonsPos();
	FLOATRECT GetRect();
public:
	CPasek();
	void SetCzas(__int64 aCzas) { Czas = aCzas; }
	void SetElementCount(int aElementCount) { ElementCount = aElementCount; }
	void Render();
	void FrameMove(DWORD dwCzas, DWORD dwDeltaCzas);
	void Wysun(); // Wysuwa lub wsuwa
	void MouseMove(WORD x, WORD y);
	bool MouseDown(WORD x, WORD y, SCREENMOUSEBUTTON button);
	bool MouseUp(WORD x, WORD y, SCREENMOUSEBUTTON button);
	bool GetClicked(int aIndex);
	bool GetDown(int aIndex) { return Buttons[aIndex].GetDown(); }
	float GetHeight() { return Height; }
};

struct RUNINFO
{
	enum RUNINFOTYPE
	{
		RUNINFOTYPE_NONE,
		RUNINFOTYPE_NEW,
		RUNINFOTYPE_OPEN
	} Type;
	std::string FileName;
	int Count;

	RUNINFO() { Type = RUNINFOTYPE_NONE; }
};

// Klasa reprezentuje ekran gry oraz sam gr
class CGameScreen : public CScreen
{
protected:
	RUNINFO RunInfo;
	LPDIRECT3DTEXTURE9 m_texPuzzle; // Gwna tekstura przechowujca pocite puzzle
	LPDIRECT3DTEXTURE9 m_texThumbnail; // Tekstura przechowujca miniaturk
	int m_iWidth;	// Szeroko oryginalnego obrazka
	int m_iHeight;	// Wysoko  oryginalnego obrazka
	int m_iElementCount;	// dana liczba elementw (niedokadnie respektowana!)
	int m_iElementCountX;	// Liczba kolumn  elementw
	int m_iElementCountY;	// Liczba wierszy elementw
	int m_iElementWidth;	// Szeroko pojedynczego elementu
	int m_iElementHeight;	// Wysoko  pojedynczego elementu
	int m_iPuzzleWidth;		// Potrzebna szeroko tekstury z pocitymi puzzlami
	int m_iPuzzleHeight;	// Potrzebna wysoko  tekstury z pocitymy pozzlami
	int m_iPuzzleA;			// Dugo boku kwadratu bdcego faktycznym rozmiarem tekstury z pocitymi puzzlami (potga dwjki)
	array1d<int>* m_aMatrixX;	// Numery pikseli, od ktrych rozpoczynaj si kolumny o kolejnych indeksach
	array1d<int>* m_aMatrixY;	// Numery pikseli, od ktrych rozpoczynaj si wiersze o kolejnych indeksach
	array2d<WORD>* m_aMatrix;	// Indeks elementu, do ktrego przynaley kady piksel obrazka
	array2d<BOOLPAIR>* m_aWypustki;	// Dla kadego elementu okrelone, czy kolejno po swojej prawej i dolnej stronie ma wypustki (true), czy "wpustki" (false)
	int ThumbnailWidth ; // Szeroko obrazu miniaturki (nie musi wypenia caej powierzchni tekstury)
	int ThumbnailHeight; // Wysoko  obrazu miniaturki (nie musi wypenia caej powierzchni tekstury)
	__int64 Czas;

	float m_fDesktopWidth;
	float m_fDesktopHeight;
	float m_fScrollX;
	float m_fScrollY;

	ARROWKEYS m_ArrowKeys;
	bool m_bAlphaMode1; // Klawisz [Ctrl]
	bool m_bAlphaMode2; // Przycisk na pasku
	bool m_bThumbnailMode1; // Klawisz [Tab]
	bool m_bThumbnailMode2; // Przycisk na pasku

	GAMEELEMENTVECTOR m_vElements; // Przenoszony jest zawsze ostatni element wektora
	bool m_bMoving; // Czy ostatni element wektora jest przenoszony?
	float m_fMoveX; // Offset X kliknicia podczas przenoszenia
	float m_fMoveY; // Offset Y kliknicia podczas przenoszenia

	DWORD WinCzas;
	bool bWin; // Czy gra zostaa ukoczona (wygrana, puzzle cae uoone)

	bool Close; // Czy gra powinna zakoczy dziaanie
	CPasek Pasek;
	CMessage *Message;

	void Load_InitializeDevIL(ILuint ImageNames[2]);
	void Load_FinalizeDevIL(ILuint ImageNames[2]);
	void Load_LoadImage(ILuint ImageNames[2], const char* lpszFileName);
	void Load_GetImageSize();
	void Load_CreateThumbnail(ILuint ImageNames[2]);
	void Load_GetPixels(ILuint ImageNames[2], array2d<IMAGECOLOR>* pPixels);
	void Load_Compute();
	void Load_ComputeMatrices();
	void Load_FillWypustki();
	void Load_FillMatrix();
	float Load_Odleglosc(float x1, float y1, float x2, float y2);
	void Load_CreateTexture();
	void Load_FillTexture(array2d<IMAGECOLOR>* pPixels);
	bool Load_Belong(int x, int y, WORD index);
	DWORD Load_GetColor(array2d<IMAGECOLOR>* pPixels, int x, int y, int light);
	void Load_CreateElements();

    void RenderElement(GAMEELEMENT element, float offsetx, float offsety, bool bShadow);
	void RenderThumbnail();
	int GetElementWidth(int eix)  { return (eix == m_iElementCountX-1) ? m_iWidth -(*m_aMatrixX)[eix] : (*m_aMatrixX)[eix+1]-(*m_aMatrixX)[eix]; }
	int GetElementHeight(int eiy) { return (eiy == m_iElementCountY-1) ? m_iHeight-(*m_aMatrixY)[eiy] : (*m_aMatrixY)[eiy+1]-(*m_aMatrixY)[eiy]; }

	bool Collision(float x, float y, GAMEELEMENT element); // Sprawdza, czy pozycja kliknicia koliduje z elementem
	void Element_MoveToBack(GAMEELEMENTVECTOR::iterator it); // Przenosi element na koniec wektora elementw

	int Index2EX(int index) { return index % m_iElementCountX; } // Przeksztaca indeks elementu na jego numer kolumny
	int Index2EY(int index) { return index / m_iElementCountX; } // Przeksztaca indeks elementu na jego numer wiersza

	int Snap(); // Dosuwa upuszczany element (czyli zawsze ostatni) i zwraca indeks elementu, do ktrego zosta dosunity lub -1 w przypadku niedosunicia

	int EX2MX(int ex) { return (*m_aMatrixX)[ex]; } // Przeksztaca numer elementu na jego pozycje na macierzy Matrix
	int EY2MY(int ey) { return (*m_aMatrixY)[ey]; } // Przeksztaca numer elementu na jego pozycje na macierzy Matrix
	void Element_FromToBack(); // Przesuwa najbliszy element na spd
	void Element_MoveToBegin(int index); // Przesuwa wskazany element na pocztek wektora
	void Shake();
	bool GetAlphaMode() { return m_bAlphaMode1 || m_bAlphaMode2; }
	bool GetThumbnailMode() { return m_bThumbnailMode1 || m_bThumbnailMode2; }
	void AskForExit();
	bool Win(); // Sprawdza, czy cale puzzle zostay uoone
	void DoWin(); // Reaguje na zakoczenie uoenia puzzli
	void DoWinEffect(float Progress);
	void DoWinEffectReset();
	void LimitScroll();

	// * FUNKCJE DO ZAPISYWANIE STANU GRY * //
	void Save(); // Zapisuje gr (funkcja gwna)
	void SaveMatrices(gzFile file);
	void SaveWypustki(gzFile file);
	void SaveMatrix(gzFile file);
	void SaveThumbnail(gzFile file);
	void SaveTexture(gzFile file);
	void SaveElements(gzFile file);
	void SaveGetFileName(char* lpszFileName);

	// * FUNKCJE DO ODCZYTYWANIA STANU GRY * //
	void Load2(); // Odczytuje gr (funkcja gwna)
	void Load2Matrices(gzFile file);
	void Load2Wypustki(gzFile file);
	void Load2Matrix(gzFile file);
	void Load2Thumbnail(gzFile file);
	void Load2Texture(gzFile file);
	void Load2Elements(gzFile file);

public:
	CGameScreen(HWND hWnd, LPDIRECT3DDEVICE9 pDevice);
	~CGameScreen();
	void SetRunInfo(RUNINFO aRunInfo) { RunInfo = aRunInfo; }

	void InitDeviceObjects();
	void RestoreDeviceObjects();
	void InvalidateDeviceObjects();
	void DeleteDeviceObjects();
	void Render();
	void FrameMove(DWORD dwCzas, DWORD dwDeltaCzas);

	void KeyDown(WPARAM wKey);
	void KeyUp(WPARAM wKey);
	void Char(WPARAM wKey);
	void MouseMove(WORD x, WORD y);
	void MouseDown(WORD x, WORD y, SCREENMOUSEBUTTON button);
	void MouseUp(WORD x, WORD y, SCREENMOUSEBUTTON button);

	int GetState();
	DWORD GetBkgColor();
};
