////////////////////////////////////////////////////////////////
// Autor : Regedit
// Data  : 2003-06-16
// Opis  : Ekran intro
////////////////////////////////////////////////////////////////

#include <windows.h>
#include <d3dx9.h>
#include "Error.h"
#include "Scr.h"
#include "intro.h"

#define FILE_WARSZTAT "Gfx\\Warsztat.png"

const float WARSZTAT_WIDTH = 128.0f;
const float WARSZTAT_HEIGHT = 128.0f;
const float WARSZTAT_SPACE = 0.0f;

CIntroScreen::CIntroScreen(HWND hWnd, LPDIRECT3DDEVICE9 pDevice) : CScreen(hWnd, pDevice)
{
	m_pWarsztatTexture = NULL;
	m_dwStartCzas = 0;
	m_bKoniec = false;
}

CIntroScreen::~CIntroScreen()
{
}

void CIntroScreen::InitDeviceObjects()
{
	HRESULT hr;

	if (FAILED(hr = m_pDevice->CreateVertexBuffer(
		INTROVERTEXCOUNT*sizeof(MYVERTEX), 0,
		FVF_MYVERTEX, D3DPOOL_MANAGED, &m_pVB, NULL)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona utworzy bufora wierzchokw dla intra", __FILE__, __LINE__, hr);

	if (FAILED(hr = D3DXCreateTextureFromFile(m_pDevice, FILE_WARSZTAT, &m_pWarsztatTexture)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona wczyta obrazka z logo Warsztatu", __FILE__, __LINE__, hr);
}

void CIntroScreen::DeleteDeviceObjects()
{
	if (m_pWarsztatTexture)
	{
		m_pWarsztatTexture->Release();
		m_pWarsztatTexture = NULL;
	}
	m_pVB->Release();
}

void CIntroScreen::RestoreDeviceObjects()
{
	HRESULT hr;

	// Tekstura
	RECT rect = {0, 0, 256, 256};
	if (FAILED(hr = m_pDevice->CreateTexture(rect.right-rect.left, rect.bottom-rect.top, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &m_pTexture, NULL)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona utworzy tekstury dla intra", __FILE__, __LINE__, hr);

	LPDIRECT3DSURFACE9 pOldRenderTarget, pSurfDest;
	m_pDevice->GetRenderTarget(0, &pOldRenderTarget);
	m_pTexture->GetSurfaceLevel(0, &pSurfDest);
	m_pDevice->SetRenderTarget(0, pSurfDest);

	if (FAILED(hr = m_pDevice->Clear(0, NULL, D3DCLEAR_TARGET, 0x00000000, 1.0f, 0L)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona wyczyci tekstury intra", __FILE__, __LINE__, hr);

	LOGFONT lf = {48, 0, 0, 0, FW_BOLD, false, false, false,
		ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
		DEFAULT_QUALITY, FF_DONTCARE, "Verdana"};
	LPD3DXFONT pFont;
	if (FAILED(hr = D3DXCreateFontIndirect(m_pDevice, &lf, &pFont)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona utworzy czcionki do intra", __FILE__, __LINE__, hr);
	m_pDevice->BeginScene();
	pFont->Begin();
	rect.top = 0; rect.bottom = 127;
	pFont->DrawTextA("ProgrameX", -1, &rect, DT_CENTER | DT_VCENTER | DT_NOCLIP, 0xffffffff);
	rect.top = 128; rect.bottom = 255;
	pFont->DrawTextA("prezentuje", -1, &rect, DT_CENTER | DT_VCENTER | DT_NOCLIP, 0xffffffff);
	pFont->End();
	m_pDevice->EndScene();
	pFont->Release();

	m_pDevice->SetRenderTarget(0, pOldRenderTarget);
	pSurfDest->Release();
	pOldRenderTarget->Release();
}

void CIntroScreen::InvalidateDeviceObjects()
{
	m_pTexture->Release();
}

void CIntroScreen::Render()
{
	// Logo Warsztat
	RenderWarsztat();

	// Intro waciwe
	m_pDevice->SetTexture(0, m_pTexture);
	m_pDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
	m_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	m_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	m_pDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	m_pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
	m_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	m_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
	m_pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	m_pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);

	m_pDevice->SetStreamSource(0, m_pVB, 0, sizeof(MYVERTEX));
	m_pDevice->SetFVF(FVF_MYVERTEX);
	m_pDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, INTROVERTEXCOUNT/3);
}

void CIntroScreen::FrameMove(DWORD dwCzas, DWORD dwDeltaCzas)
{
	Czas = dwCzas;

	HRESULT hr;

	if (m_dwStartCzas == 0)
		m_dwStartCzas = dwCzas;
	if (dwCzas > m_dwStartCzas + INTROTIME)
		m_bKoniec = true;

	// Wypenienie bufora wierzchokw
	MYVERTEX* v;
	if (FAILED(hr = m_pVB->Lock(0, 0, (void**)&v, D3DLOCK_DISCARD)))
		throw CError(ERRTYPE_DIRECTX, "Nie mona zablokowa bufora wierzchokw intra", __FILE__, __LINE__, hr);

	int a, b;
	DWORD color;
	BYTE byte;
	float ty1, ty2;
	float x1, x2;
	float y1, y2;
	float deltax;

	// - Grna i dolna
	for (a = 0; a < 2; a++)
	{
		ty1 = (a == 0) ? 0.0f : 0.5f;
		ty2 = (a == 0) ? 0.5f : 1.0f;

		// -- Pierwsza, druga i trzecia
		for (b = 0; b < 3; b++)
		{
			switch (b)
			{
			case 0: byte = 96; break;
			case 1: byte = 128; break;
			case 2: byte = 255; break;
			}
			byte = (BYTE)(sinf((dwCzas-m_dwStartCzas)*D3DX_PI/INTROTIME) * byte);
			color = D3DCOLOR_ARGB(byte, 255, 255, 255);

			deltax = ((dwCzas-m_dwStartCzas) / (float)INTROTIME - 0.5f) * (3.0f-b)*150.0f;
			
			x1 = SCRWIDTH/2.0f - SCRWIDTH/4.0f - (2.0f-b)*150.0f - deltax;
			x2 = SCRWIDTH/2.0f + SCRWIDTH/4.0f + (2.0f-b)*150.0f - deltax;

			y1 = 150.0f - 100.0f - (2-b)*100.0f + ((a==0.0f)?0.0f:300.0f);
			y2 = 150.0f + 100.0f + (2-b)*100.0f + ((a==0.0f)?0.0f:300.0f);

			if (a == 1)
			{
				x1 = (x1-400.0f) / 2.0f + 400.0f;
				x2 = (x2-400.0f) / 2.0f + 400.0f;
				y1 = (y1-300.0f) / 2.0f + 375.0f;
				y2 = (y2-300.0f) / 2.0f + 375.0f;
			}

			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+0].pos = D3DXVECTOR3(x1, y1, 0.0f);
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+0].rhw = 1.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+0].color = color;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+0].tx = 0.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+0].ty = ty1;

			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+1].pos = D3DXVECTOR3(x1, y2, 0.0f);
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+1].rhw = 1.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+1].color = color;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+1].tx = 0.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+1].ty = ty2;

			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+2].pos = D3DXVECTOR3(x2, y1, 0.0f);
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+2].rhw = 1.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+2].color = color;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+2].tx = 1.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+2].ty = ty1;

			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+3].pos = D3DXVECTOR3(x1, y2, 0.0f);
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+3].rhw = 1.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+3].color = color;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+3].tx = 0.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+3].ty = ty2;
	
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+4].pos = D3DXVECTOR3(x2, y1, 0.0f);
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+4].rhw = 1.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+4].color = color;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+4].tx = 1.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+4].ty = ty1;

			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+5].pos = D3DXVECTOR3(x2, y2, 0.0f);
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+5].rhw = 1.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+5].color = color;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+5].tx = 1.0f;
			v[a*INTROVERTEXCOUNT/2+b*INTROVERTEXCOUNT/(2*3)+5].ty = ty2;
		}
	}

	m_pVB->Unlock();
}

void CIntroScreen::KeyDown(WPARAM wKey)
{
	m_bKoniec = true;
}

void CIntroScreen::MouseDown(WORD x, WORD y, SCREENMOUSEBUTTON button)
{
	m_bKoniec = true;
}

int CIntroScreen::GetState()
{
	if (m_bKoniec)
		return SCREENSTATE_INTROEND;
	else
		return SCREENSTATE_NONE;
}

void CIntroScreen::RenderWarsztat()
{
	MYVERTEX v[4];

	v[0].color = v[1].color = v[2].color = v[3].color = D3DCOLOR_ARGB(
		static_cast<BYTE>(
			sinf( (Czas-m_dwStartCzas) * D3DX_PI/INTROTIME ) * 255
			), 255, 255, 255
		);
	v[0].pos.z = v[1].pos.z = v[2].pos.z = v[3].pos.z = 0.0f;
	v[0].rhw = v[1].rhw = v[2].rhw = v[3].rhw = 1.0f;
	v[0].pos.x = v[1].pos.x = SCRWIDTH-WARSZTAT_SPACE-WARSZTAT_WIDTH;
	v[2].pos.x = v[3].pos.x = SCRWIDTH-WARSZTAT_SPACE;
	v[0].pos.y = v[2].pos.y = SCRHEIGHT-WARSZTAT_SPACE-WARSZTAT_HEIGHT;
	v[1].pos.y = v[3].pos.y = SCRHEIGHT-WARSZTAT_SPACE;
	v[0].tx = v[1].tx = 0.0f;
	v[2].tx = v[3].tx = 1.0f;
	v[0].ty = v[2].ty = 0.0f;
	v[1].ty = v[3].ty = 1.0f;

	m_pDevice->SetTexture(0, m_pWarsztatTexture);
	m_pDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
	m_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	m_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	m_pDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	m_pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
	m_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	m_pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
	m_pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	m_pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);

	m_pDevice->SetFVF(FVF_MYVERTEX);
	m_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, v, sizeof(MYVERTEX));
}