////////////////////////////////////////////////////////////////
// Autor : Regedit
// Data  : 2003-07-13
// Opis  : Plik gwny loadera do gry Puzzle
////////////////////////////////////////////////////////////////
#include <windows.h>
#include <stdio.h> // sprintf()
#include <stdlib.h> // atoi()
#include <string>
#include "resource.h"

////////////////////////////////////////////////////////////////
// Stae

const char CLASS_NAME[] = "PROGRAMEX_PUZZLE";
const char TITLE[] = "ProgrameX Puzzle";
const UINT WM_DIALOG = WM_USER + 10;
const int FORMAT_COUNT = 23;
const char FORMATS[][FORMAT_COUNT] = {
	"bmp", "cut", "dcx", "dds",
	"ico", "jpg", "lbm", "lif",
	"mdl", "pcd", "pcx", "pic",
	"png", "pnm", "psd", "psp",
	"raw", "sgi", "tga", "tif",
	"wal", "act", "pal"
};

////////////////////////////////////////////////////////////////
// Zmienne globalne

HINSTANCE g_hInstance = NULL;
char* lpszOpenDialogFilter = NULL;

////////////////////////////////////////////////////////////////
// Forwarding

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

////////////////////////////////////////////////////////////////
// ProgrameX Namespace

namespace Px
{
	// Zwraca true, jeli plik o podanej nazwie istnieje i nie jest katalogiem
	bool FileExists(const std::string &FileName)
	{
		// acuch pusty
		if (FileName.empty())
			return false;
		// szukanie pliku
		DWORD Attr = GetFileAttributes(FileName.c_str());
		// pliku nie ma
		if (Attr == INVALID_FILE_ATTRIBUTES)
			return false;
		// to jest katalog
		if ((Attr & FILE_ATTRIBUTE_DIRECTORY) != 0)
			return false;
		// koniec - to jest plik :D
        return true;
	}
}

////////////////////////////////////////////////////////////////
// Funkcje

void InitOpenDialogFilter()
{
	// Zmienne
	size_t i;
	std::string s;

	// Dodanie filtra gwnego
	s += "Wszystkie obrazki";
	s += '\0';
	for (i = 0; i < FORMAT_COUNT; i++)
	{
		s += "*.";
		s += FORMATS[i];
		if (i < FORMAT_COUNT-1)
			s += ';';
	}
	s += '\0';

	// Dodanie listy formatw
	for (i = 0; i < FORMAT_COUNT; i++)
	{
		s += "Obrazki .";
		s += FORMATS[i];
		s += '\0';
		s += "*.";
		s += FORMATS[i];
		s += '\0';
	}

	// Dodanie filrta koczcego
	s += "Wszystkie pliki";
	s += '\0';
	s += "*.*";
	s += '\0';

	// Przepisanie do lpcstr :)
	lpszOpenDialogFilter = new char[s.size()+1];
	for (i = 0; i < s.size(); i++)
		lpszOpenDialogFilter[i] = s[i];
	lpszOpenDialogFilter[s.size()] = '\0';
}

std::string FileNameToName(char* lpszFileName)
{
	char drv[MAX_PATH], path[MAX_PATH], name[MAX_PATH], ext[MAX_PATH];
	_splitpath(lpszFileName, drv, path, name, ext);
	return name;
}

void RefreshFileList(HWND hWnd)
{
	HWND hList = GetDlgItem(hWnd, IDC_LIST);
	SendMessage(hList, LB_RESETCONTENT, 0, 0);

	WIN32_FIND_DATA win32fd;
	HANDLE Handle = FindFirstFile("*.pzz", &win32fd);
	if (Handle != INVALID_HANDLE_VALUE)
	{
		do {
			SendMessage(hList, LB_ADDSTRING, 0, (LPARAM)(FileNameToName(win32fd.cFileName).c_str()));
		} while (FindNextFile(Handle, &win32fd));
		FindClose(Handle);
	}
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, char* lpszCmdLine, int iCmdShow)
{
	g_hInstance = hInstance;

	InitOpenDialogFilter();

	// Utworzenie klasy
	WNDCLASS WndClass;
	ZeroMemory(&WndClass, sizeof(WndClass));
	WndClass.style = CS_HREDRAW | CS_VREDRAW;
	WndClass.lpfnWndProc = WndProc;
	WndClass.hInstance = hInstance;
	WndClass.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
	WndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
	WndClass.hbrBackground = (HBRUSH)(COLOR_BTNFACE+1);
	WndClass.lpszClassName = CLASS_NAME;

	if (!RegisterClass(&WndClass))
		return 1;

	// Utworzenie okna
	HWND hWnd = CreateWindow(CLASS_NAME, TITLE, WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
		NULL, NULL, hInstance, NULL);
//	ShowWindow(hWnd, iCmdShow);
//	UpdateWindow(hWnd);

	// Ptla komunikatw
	MSG msg;
	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	delete [] lpszOpenDialogFilter;

	return static_cast<int>(msg.wParam);
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_CREATE:
		PostMessage(hWnd, WM_DIALOG, 0, 0);
		return 0;

	case WM_DIALOG:
		DialogBox(g_hInstance, "MAIN", hWnd, DlgProc);
		PostQuitMessage(0);

/*	case WM_DESTROY:
		PostQuitMessage(0);
		return 0; */
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

void Browse(HWND hWnd)
{
	char szFileName[MAX_PATH] = "";

	OPENFILENAME ofn;
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hWnd;
	ofn.lpstrFilter = lpszOpenDialogFilter;
	ofn.nFilterIndex = 1;
	ofn.lpstrFile = szFileName;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrTitle = "Przegldaj";
	ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	if (GetOpenFileName(&ofn))
		SetWindowText(GetDlgItem(hWnd, IDC_EDIT_FILENAME), ofn.lpstrFile);
}

void Warning(HWND hWnd, std::string Message)
{
	MessageBox(hWnd, Message.c_str(), "ProgrameX Puzzle", MB_OK | MB_ICONEXCLAMATION);
}

void GetEditText(HWND hWnd, int iDlgItem, std::string &str)
{
	const int BUFOR_SIZE = MAX_PATH;
	char Bufor[BUFOR_SIZE];
	int Size;

	(reinterpret_cast<int*>(Bufor))[0] = BUFOR_SIZE;
	Size = static_cast<int>(
		SendMessage(
			GetDlgItem(hWnd, iDlgItem),
			EM_GETLINE, 0,
			reinterpret_cast<LPARAM>(Bufor)
			)
		);

	str.erase();
	str.append(Bufor, Size);
}

bool DoNew(HWND hWnd, std::string &Parameters)
{
	// nazwa pliku
	std::string FileName;
	GetEditText(hWnd, IDC_EDIT_FILENAME, FileName);
	// - jeli pusta
	if (FileName == "")
	{
		Warning(hWnd, "Wpisz lub wybierz obrazek !");
		SetFocus(GetDlgItem(hWnd, IDC_EDIT_FILENAME));
		return false;
	}
	// - sprawdzenie, czy plik istnieje
	if (!Px::FileExists(FileName))
	{
		Warning(hWnd, "Podany obrazek nie istnieje !");
		SetFocus(GetDlgItem(hWnd, IDC_EDIT_FILENAME));
		return false;
	}

	// liczba elementw
	std::string ElementCount;
	GetEditText(hWnd, IDC_EDIT_COUNT, ElementCount);
	// - jeli pusta
	if (ElementCount == "")
	{
		Warning(hWnd, "Wpisz liczb elementw !");
		SetFocus(GetDlgItem(hWnd, IDC_EDIT_COUNT));
		return false;
	}
	// - przerobienie na liczb
	int iElementCount = atoi(ElementCount.c_str());
	// - zero
	if (iElementCount == 0)
	{
		Warning(hWnd, "Wpisz poprawn liczb elementw !");
		SetFocus(GetDlgItem(hWnd, IDC_EDIT_COUNT));
		return false;
	}
	// - zakres
	if ( (iElementCount < 4) || (iElementCount > 10000) )
	{
		Warning(hWnd, "Wpisz jak sensown liczb elementw !");
		SetFocus(GetDlgItem(hWnd, IDC_EDIT_COUNT));
		return false;
	}

	// Koniec - zwrcenie informacji
	Parameters = "\"--new:" + FileName + "?" + ElementCount + "\"";
	return true;
}

bool DoOpen(HWND hWnd, std::string &Parameters)
{
	HWND ListBox = GetDlgItem(hWnd, IDC_LIST);

    int ItemIndex = static_cast<int>(SendMessage(ListBox, LB_GETCURSEL, 0, 0));
	if (ItemIndex == -1)
	{
		Warning(hWnd, "Zaznacz na licie zapisan gr, ktr chcesz wczyta !");
		SetFocus(ListBox);
		return false;
	}

	char Buffer[MAX_PATH];
	SendMessage(ListBox, LB_GETTEXT, ItemIndex, reinterpret_cast<LPARAM>(Buffer));

	Parameters += "\"--open:";
	Parameters += Buffer;
	Parameters += ".pzz\"";
	return true;
}

bool OK(HWND hWnd)
{
	// Zmienne
	std::string Parameters;
	bool Result;

    // Nowe puzzle
	if (SendMessage(GetDlgItem(hWnd, IDC_RADIO1), BM_GETCHECK, 0, 0) == BST_CHECKED)
		Result = DoNew(hWnd, Parameters);
	// Otwrz zapisan gr
	else if (SendMessage(GetDlgItem(hWnd, IDC_RADIO2), BM_GETCHECK, 0, 0) == BST_CHECKED)
		Result = DoOpen(hWnd, Parameters);

	// Wreszcie uruchomienie
	if (Result && (Parameters != ""))
	{
		std::string CmdLine = "Game.exe " + Parameters;
//		MessageBox(hWnd, CmdLine.c_str(), "Test", MB_OK);
		WinExec(CmdLine.c_str(), SW_SHOWNORMAL);
	}
	return Result;
}

void Delete(HWND hWnd)
{
	HWND ListBox = GetDlgItem(hWnd, IDC_LIST);

    int ItemIndex = static_cast<int>(SendMessage(ListBox, LB_GETCURSEL, 0, 0));
	if (ItemIndex == -1)
	{
		Warning(hWnd, "Zaznacz na licie zapisan gr, ktr chcesz usun !");
		SetFocus(ListBox);
		return;
	}

	char Buffer[MAX_PATH];
	SendMessage(ListBox, LB_GETTEXT, ItemIndex, reinterpret_cast<LPARAM>(Buffer));

	std::string MessageText;
	MessageText += "Czy na pewno chcesz usun zapisan gr o nazwie \"";
	MessageText += Buffer;
	MessageText += "\" ?";
	int Response = MessageBox(hWnd, MessageText.c_str(), "Usu", MB_YESNO | MB_ICONEXCLAMATION);

	if (Response == IDYES)
	{
		std::string FileName;
		FileName += Buffer;
		FileName += ".pzz";
		DeleteFile(FileName.c_str());
		RefreshFileList(hWnd);
	}
}

void SetDialogIcon(HWND hWnd)
{
	SetClassLong(
		hWnd,
		GCL_HICON,
		reinterpret_cast<LONG>(
			LoadIcon(
				g_hInstance,
				MAKEINTRESOURCE(IDI_ICON1)
				)
			)
		);
}

BOOL CALLBACK DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		SetDialogIcon(hWnd);
		CheckRadioButton(hWnd, IDC_RADIO1, IDC_RADIO2, IDC_RADIO1);
		RefreshFileList(hWnd);
		return true;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_BUTTON_BROWSE:
			Browse(hWnd);
			break;
		case IDC_BUTTON_DELETE:
			Delete(hWnd);
			break;
		case IDC_BUTTON_REFRESH:
			RefreshFileList(hWnd);
			break;
		case IDOK:
			if (OK(hWnd))
				EndDialog(hWnd, true);
			break;
		case IDCANCEL:
			EndDialog(hWnd, false);
			break;
		}
		return true;

	case WM_CLOSE:
		EndDialog(hWnd, false);
		return true;
	}
	return false;
}
